-- The conf2 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/01/02

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_conf2 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  OutChSel : in std_logic_vector(6 downto 0);
  CO300 : in std_logic_vector(8 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO1024 : in std_logic_vector(9 downto 0);
  CO1024_dly2 : in std_logic_vector(9 downto 0);
  X_RD_DATA : in std_logic_vector(8191 downto 0); 
  ReluAll : out std_logic_vector(1511 downto 0); 
  ReluAllWrN : out std_logic; 
  ReluSum : out std_logic_vector(11 downto 0) ); 
end conv_conf2;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_conf2 is

component dotpro_conf2
   Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(7 downto 0);
  MatRomData : in std_logic_vector(2 downto 0);
  CO9is1 : in std_logic;
  CO1024is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(16 downto 0) );
end component;

component ROM_CW2
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(13 downto 0);
          DATA : Out std_logic_vector(377 downto 0) );
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal CO9_clr : std_logic;
signal W3BUSY, W3BUSY_dly1, W3BUSY_dly2, W3BUSY_dly3, W3BUSY_dly4 : std_logic;

signal mac_000, mac_001 : std_logic_vector(16 downto 0);
signal mac_002, mac_003 : std_logic_vector(16 downto 0);
signal mac_004, mac_005 : std_logic_vector(16 downto 0);
signal mac_006, mac_007 : std_logic_vector(16 downto 0);
signal mac_008, mac_009 : std_logic_vector(16 downto 0);
signal mac_010, mac_011 : std_logic_vector(16 downto 0);
signal mac_012, mac_013 : std_logic_vector(16 downto 0);
signal mac_014, mac_015 : std_logic_vector(16 downto 0);

signal truncate_000, truncate_001 : std_logic_vector(12 downto 0);
signal truncate_002, truncate_003 : std_logic_vector(12 downto 0);
signal truncate_004, truncate_005 : std_logic_vector(12 downto 0);
signal truncate_006, truncate_007 : std_logic_vector(12 downto 0);
signal truncate_008, truncate_009 : std_logic_vector(12 downto 0);
signal truncate_010, truncate_011 : std_logic_vector(12 downto 0);
signal truncate_012, truncate_013 : std_logic_vector(12 downto 0);
signal truncate_014, truncate_015 : std_logic_vector(12 downto 0);

signal Relu_000, Relu_001 : std_logic_vector(11 downto 0);
signal Relu_002, Relu_003 : std_logic_vector(11 downto 0);
signal Relu_004, Relu_005 : std_logic_vector(11 downto 0);
signal Relu_006, Relu_007 : std_logic_vector(11 downto 0);
signal Relu_008, Relu_009 : std_logic_vector(11 downto 0);
signal Relu_010, Relu_011 : std_logic_vector(11 downto 0);
signal Relu_012, Relu_013 : std_logic_vector(11 downto 0);
signal Relu_014, Relu_015 : std_logic_vector(11 downto 0);

signal PixRomData : std_logic_vector(7 downto 0);
signal CW2Addr, CW2Addr_prev : std_logic_vector(13 downto 0);
signal CW2Addr_L : std_logic_vector(3 downto 0);
signal CW2Addr_H : std_logic_vector(13 downto 0);
signal CW2Sel : std_logic_vector(6 downto 0);
signal OutChSel_dly1, OutChSel_dly2 : std_logic_vector(6 downto 0);

signal reg_000_000, reg_000_001, reg_000_002, reg_000_003, reg_000_004, reg_000_005, reg_000_006, reg_000_007: std_logic_vector(11 downto 0);

signal reg_001_000, reg_001_001, reg_001_002, reg_001_003, reg_001_004, reg_001_005, reg_001_006, reg_001_007: std_logic_vector(11 downto 0);

signal reg_002_000, reg_002_001, reg_002_002, reg_002_003, reg_002_004, reg_002_005, reg_002_006, reg_002_007: std_logic_vector(11 downto 0);

signal reg_003_000, reg_003_001, reg_003_002, reg_003_003, reg_003_004, reg_003_005, reg_003_006, reg_003_007: std_logic_vector(11 downto 0);

signal reg_004_000, reg_004_001, reg_004_002, reg_004_003, reg_004_004, reg_004_005, reg_004_006, reg_004_007: std_logic_vector(11 downto 0);

signal reg_005_000, reg_005_001, reg_005_002, reg_005_003, reg_005_004, reg_005_005, reg_005_006, reg_005_007: std_logic_vector(11 downto 0);

signal reg_006_000, reg_006_001, reg_006_002, reg_006_003, reg_006_004, reg_006_005, reg_006_006, reg_006_007: std_logic_vector(11 downto 0);

signal reg_007_000, reg_007_001, reg_007_002, reg_007_003, reg_007_004, reg_007_005, reg_007_006, reg_007_007: std_logic_vector(11 downto 0);

signal reg_008_000, reg_008_001, reg_008_002, reg_008_003, reg_008_004, reg_008_005, reg_008_006, reg_008_007: std_logic_vector(11 downto 0);

signal reg_009_000, reg_009_001, reg_009_002, reg_009_003, reg_009_004, reg_009_005, reg_009_006, reg_009_007: std_logic_vector(11 downto 0);

signal reg_010_000, reg_010_001, reg_010_002, reg_010_003, reg_010_004, reg_010_005, reg_010_006, reg_010_007: std_logic_vector(11 downto 0);

signal reg_011_000, reg_011_001, reg_011_002, reg_011_003, reg_011_004, reg_011_005, reg_011_006, reg_011_007: std_logic_vector(11 downto 0);

signal reg_012_000, reg_012_001, reg_012_002, reg_012_003, reg_012_004, reg_012_005, reg_012_006, reg_012_007: std_logic_vector(11 downto 0);

signal reg_013_000, reg_013_001, reg_013_002, reg_013_003, reg_013_004, reg_013_005, reg_013_006, reg_013_007: std_logic_vector(11 downto 0);

signal reg_014_000, reg_014_001, reg_014_002, reg_014_003, reg_014_004, reg_014_005, reg_014_006, reg_014_007: std_logic_vector(11 downto 0);

signal reg_015_000, reg_015_001, reg_015_002, reg_015_003, reg_015_004, reg_015_005, reg_015_006, reg_015_007: std_logic_vector(11 downto 0);

signal enb_000, enb_001, enb_002, enb_003, enb_004, enb_005, enb_006, enb_007: std_logic;

signal CO9is1, CO1024is0 : std_logic;
signal CW2Data : std_logic_vector(377 downto 0);
signal coeff_000, coeff_001: std_logic_vector(2 downto 0);
signal coeff_002, coeff_003: std_logic_vector(2 downto 0);
signal coeff_004, coeff_005: std_logic_vector(2 downto 0);
signal coeff_006, coeff_007: std_logic_vector(2 downto 0);
signal coeff_008, coeff_009: std_logic_vector(2 downto 0);
signal coeff_010, coeff_011: std_logic_vector(2 downto 0);
signal coeff_012, coeff_013: std_logic_vector(2 downto 0);
signal coeff_014, coeff_015: std_logic_vector(2 downto 0);

begin

CO9is1 <= '1' when CO9 = 1 else '0'; 
--CO1024is0 <= '1' when CO1024 = 1 else '0'; -- for lay15 one more dly
CO1024is0 <= '1' when CO1024 = 0 else '0'; -- for ex2 no more dly

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		OutChSel_dly2 <= (others => '0');
		OutChSel_dly1 <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		OutChSel_dly2 <= OutChSel_dly1;
		OutChSel_dly1 <= OutChSel;
	end if;  
end process;

CW2Sel <= OutChSel_dly2;

---------------------------------------------------------
  PixRomData <= 
X_RD_DATA( 8191 downto 8184 ) when CO1024_dly2 = "0000000000" else 
X_RD_DATA( 8183 downto 8176 ) when CO1024_dly2 = "0000000001" else 
X_RD_DATA( 8175 downto 8168 ) when CO1024_dly2 = "0000000010" else 
X_RD_DATA( 8167 downto 8160 ) when CO1024_dly2 = "0000000011" else 
X_RD_DATA( 8159 downto 8152 ) when CO1024_dly2 = "0000000100" else 
X_RD_DATA( 8151 downto 8144 ) when CO1024_dly2 = "0000000101" else 
X_RD_DATA( 8143 downto 8136 ) when CO1024_dly2 = "0000000110" else 
X_RD_DATA( 8135 downto 8128 ) when CO1024_dly2 = "0000000111" else 
X_RD_DATA( 8127 downto 8120 ) when CO1024_dly2 = "0000001000" else 
X_RD_DATA( 8119 downto 8112 ) when CO1024_dly2 = "0000001001" else 
X_RD_DATA( 8111 downto 8104 ) when CO1024_dly2 = "0000001010" else 
X_RD_DATA( 8103 downto 8096 ) when CO1024_dly2 = "0000001011" else 
X_RD_DATA( 8095 downto 8088 ) when CO1024_dly2 = "0000001100" else 
X_RD_DATA( 8087 downto 8080 ) when CO1024_dly2 = "0000001101" else 
X_RD_DATA( 8079 downto 8072 ) when CO1024_dly2 = "0000001110" else 
X_RD_DATA( 8071 downto 8064 ) when CO1024_dly2 = "0000001111" else 
X_RD_DATA( 8063 downto 8056 ) when CO1024_dly2 = "0000010000" else 
X_RD_DATA( 8055 downto 8048 ) when CO1024_dly2 = "0000010001" else 
X_RD_DATA( 8047 downto 8040 ) when CO1024_dly2 = "0000010010" else 
X_RD_DATA( 8039 downto 8032 ) when CO1024_dly2 = "0000010011" else 
X_RD_DATA( 8031 downto 8024 ) when CO1024_dly2 = "0000010100" else 
X_RD_DATA( 8023 downto 8016 ) when CO1024_dly2 = "0000010101" else 
X_RD_DATA( 8015 downto 8008 ) when CO1024_dly2 = "0000010110" else 
X_RD_DATA( 8007 downto 8000 ) when CO1024_dly2 = "0000010111" else 
X_RD_DATA( 7999 downto 7992 ) when CO1024_dly2 = "0000011000" else 
X_RD_DATA( 7991 downto 7984 ) when CO1024_dly2 = "0000011001" else 
X_RD_DATA( 7983 downto 7976 ) when CO1024_dly2 = "0000011010" else 
X_RD_DATA( 7975 downto 7968 ) when CO1024_dly2 = "0000011011" else 
X_RD_DATA( 7967 downto 7960 ) when CO1024_dly2 = "0000011100" else 
X_RD_DATA( 7959 downto 7952 ) when CO1024_dly2 = "0000011101" else 
X_RD_DATA( 7951 downto 7944 ) when CO1024_dly2 = "0000011110" else 
X_RD_DATA( 7943 downto 7936 ) when CO1024_dly2 = "0000011111" else 
X_RD_DATA( 7935 downto 7928 ) when CO1024_dly2 = "0000100000" else 
X_RD_DATA( 7927 downto 7920 ) when CO1024_dly2 = "0000100001" else 
X_RD_DATA( 7919 downto 7912 ) when CO1024_dly2 = "0000100010" else 
X_RD_DATA( 7911 downto 7904 ) when CO1024_dly2 = "0000100011" else 
X_RD_DATA( 7903 downto 7896 ) when CO1024_dly2 = "0000100100" else 
X_RD_DATA( 7895 downto 7888 ) when CO1024_dly2 = "0000100101" else 
X_RD_DATA( 7887 downto 7880 ) when CO1024_dly2 = "0000100110" else 
X_RD_DATA( 7879 downto 7872 ) when CO1024_dly2 = "0000100111" else 
X_RD_DATA( 7871 downto 7864 ) when CO1024_dly2 = "0000101000" else 
X_RD_DATA( 7863 downto 7856 ) when CO1024_dly2 = "0000101001" else 
X_RD_DATA( 7855 downto 7848 ) when CO1024_dly2 = "0000101010" else 
X_RD_DATA( 7847 downto 7840 ) when CO1024_dly2 = "0000101011" else 
X_RD_DATA( 7839 downto 7832 ) when CO1024_dly2 = "0000101100" else 
X_RD_DATA( 7831 downto 7824 ) when CO1024_dly2 = "0000101101" else 
X_RD_DATA( 7823 downto 7816 ) when CO1024_dly2 = "0000101110" else 
X_RD_DATA( 7815 downto 7808 ) when CO1024_dly2 = "0000101111" else 
X_RD_DATA( 7807 downto 7800 ) when CO1024_dly2 = "0000110000" else 
X_RD_DATA( 7799 downto 7792 ) when CO1024_dly2 = "0000110001" else 
X_RD_DATA( 7791 downto 7784 ) when CO1024_dly2 = "0000110010" else 
X_RD_DATA( 7783 downto 7776 ) when CO1024_dly2 = "0000110011" else 
X_RD_DATA( 7775 downto 7768 ) when CO1024_dly2 = "0000110100" else 
X_RD_DATA( 7767 downto 7760 ) when CO1024_dly2 = "0000110101" else 
X_RD_DATA( 7759 downto 7752 ) when CO1024_dly2 = "0000110110" else 
X_RD_DATA( 7751 downto 7744 ) when CO1024_dly2 = "0000110111" else 
X_RD_DATA( 7743 downto 7736 ) when CO1024_dly2 = "0000111000" else 
X_RD_DATA( 7735 downto 7728 ) when CO1024_dly2 = "0000111001" else 
X_RD_DATA( 7727 downto 7720 ) when CO1024_dly2 = "0000111010" else 
X_RD_DATA( 7719 downto 7712 ) when CO1024_dly2 = "0000111011" else 
X_RD_DATA( 7711 downto 7704 ) when CO1024_dly2 = "0000111100" else 
X_RD_DATA( 7703 downto 7696 ) when CO1024_dly2 = "0000111101" else 
X_RD_DATA( 7695 downto 7688 ) when CO1024_dly2 = "0000111110" else 
X_RD_DATA( 7687 downto 7680 ) when CO1024_dly2 = "0000111111" else 
X_RD_DATA( 7679 downto 7672 ) when CO1024_dly2 = "0001000000" else 
X_RD_DATA( 7671 downto 7664 ) when CO1024_dly2 = "0001000001" else 
X_RD_DATA( 7663 downto 7656 ) when CO1024_dly2 = "0001000010" else 
X_RD_DATA( 7655 downto 7648 ) when CO1024_dly2 = "0001000011" else 
X_RD_DATA( 7647 downto 7640 ) when CO1024_dly2 = "0001000100" else 
X_RD_DATA( 7639 downto 7632 ) when CO1024_dly2 = "0001000101" else 
X_RD_DATA( 7631 downto 7624 ) when CO1024_dly2 = "0001000110" else 
X_RD_DATA( 7623 downto 7616 ) when CO1024_dly2 = "0001000111" else 
X_RD_DATA( 7615 downto 7608 ) when CO1024_dly2 = "0001001000" else 
X_RD_DATA( 7607 downto 7600 ) when CO1024_dly2 = "0001001001" else 
X_RD_DATA( 7599 downto 7592 ) when CO1024_dly2 = "0001001010" else 
X_RD_DATA( 7591 downto 7584 ) when CO1024_dly2 = "0001001011" else 
X_RD_DATA( 7583 downto 7576 ) when CO1024_dly2 = "0001001100" else 
X_RD_DATA( 7575 downto 7568 ) when CO1024_dly2 = "0001001101" else 
X_RD_DATA( 7567 downto 7560 ) when CO1024_dly2 = "0001001110" else 
X_RD_DATA( 7559 downto 7552 ) when CO1024_dly2 = "0001001111" else 
X_RD_DATA( 7551 downto 7544 ) when CO1024_dly2 = "0001010000" else 
X_RD_DATA( 7543 downto 7536 ) when CO1024_dly2 = "0001010001" else 
X_RD_DATA( 7535 downto 7528 ) when CO1024_dly2 = "0001010010" else 
X_RD_DATA( 7527 downto 7520 ) when CO1024_dly2 = "0001010011" else 
X_RD_DATA( 7519 downto 7512 ) when CO1024_dly2 = "0001010100" else 
X_RD_DATA( 7511 downto 7504 ) when CO1024_dly2 = "0001010101" else 
X_RD_DATA( 7503 downto 7496 ) when CO1024_dly2 = "0001010110" else 
X_RD_DATA( 7495 downto 7488 ) when CO1024_dly2 = "0001010111" else 
X_RD_DATA( 7487 downto 7480 ) when CO1024_dly2 = "0001011000" else 
X_RD_DATA( 7479 downto 7472 ) when CO1024_dly2 = "0001011001" else 
X_RD_DATA( 7471 downto 7464 ) when CO1024_dly2 = "0001011010" else 
X_RD_DATA( 7463 downto 7456 ) when CO1024_dly2 = "0001011011" else 
X_RD_DATA( 7455 downto 7448 ) when CO1024_dly2 = "0001011100" else 
X_RD_DATA( 7447 downto 7440 ) when CO1024_dly2 = "0001011101" else 
X_RD_DATA( 7439 downto 7432 ) when CO1024_dly2 = "0001011110" else 
X_RD_DATA( 7431 downto 7424 ) when CO1024_dly2 = "0001011111" else 
X_RD_DATA( 7423 downto 7416 ) when CO1024_dly2 = "0001100000" else 
X_RD_DATA( 7415 downto 7408 ) when CO1024_dly2 = "0001100001" else 
X_RD_DATA( 7407 downto 7400 ) when CO1024_dly2 = "0001100010" else 
X_RD_DATA( 7399 downto 7392 ) when CO1024_dly2 = "0001100011" else 
X_RD_DATA( 7391 downto 7384 ) when CO1024_dly2 = "0001100100" else 
X_RD_DATA( 7383 downto 7376 ) when CO1024_dly2 = "0001100101" else 
X_RD_DATA( 7375 downto 7368 ) when CO1024_dly2 = "0001100110" else 
X_RD_DATA( 7367 downto 7360 ) when CO1024_dly2 = "0001100111" else 
X_RD_DATA( 7359 downto 7352 ) when CO1024_dly2 = "0001101000" else 
X_RD_DATA( 7351 downto 7344 ) when CO1024_dly2 = "0001101001" else 
X_RD_DATA( 7343 downto 7336 ) when CO1024_dly2 = "0001101010" else 
X_RD_DATA( 7335 downto 7328 ) when CO1024_dly2 = "0001101011" else 
X_RD_DATA( 7327 downto 7320 ) when CO1024_dly2 = "0001101100" else 
X_RD_DATA( 7319 downto 7312 ) when CO1024_dly2 = "0001101101" else 
X_RD_DATA( 7311 downto 7304 ) when CO1024_dly2 = "0001101110" else 
X_RD_DATA( 7303 downto 7296 ) when CO1024_dly2 = "0001101111" else 
X_RD_DATA( 7295 downto 7288 ) when CO1024_dly2 = "0001110000" else 
X_RD_DATA( 7287 downto 7280 ) when CO1024_dly2 = "0001110001" else 
X_RD_DATA( 7279 downto 7272 ) when CO1024_dly2 = "0001110010" else 
X_RD_DATA( 7271 downto 7264 ) when CO1024_dly2 = "0001110011" else 
X_RD_DATA( 7263 downto 7256 ) when CO1024_dly2 = "0001110100" else 
X_RD_DATA( 7255 downto 7248 ) when CO1024_dly2 = "0001110101" else 
X_RD_DATA( 7247 downto 7240 ) when CO1024_dly2 = "0001110110" else 
X_RD_DATA( 7239 downto 7232 ) when CO1024_dly2 = "0001110111" else 
X_RD_DATA( 7231 downto 7224 ) when CO1024_dly2 = "0001111000" else 
X_RD_DATA( 7223 downto 7216 ) when CO1024_dly2 = "0001111001" else 
X_RD_DATA( 7215 downto 7208 ) when CO1024_dly2 = "0001111010" else 
X_RD_DATA( 7207 downto 7200 ) when CO1024_dly2 = "0001111011" else 
X_RD_DATA( 7199 downto 7192 ) when CO1024_dly2 = "0001111100" else 
X_RD_DATA( 7191 downto 7184 ) when CO1024_dly2 = "0001111101" else 
X_RD_DATA( 7183 downto 7176 ) when CO1024_dly2 = "0001111110" else 
X_RD_DATA( 7175 downto 7168 ) when CO1024_dly2 = "0001111111" else 
X_RD_DATA( 7167 downto 7160 ) when CO1024_dly2 = "0010000000" else 
X_RD_DATA( 7159 downto 7152 ) when CO1024_dly2 = "0010000001" else 
X_RD_DATA( 7151 downto 7144 ) when CO1024_dly2 = "0010000010" else 
X_RD_DATA( 7143 downto 7136 ) when CO1024_dly2 = "0010000011" else 
X_RD_DATA( 7135 downto 7128 ) when CO1024_dly2 = "0010000100" else 
X_RD_DATA( 7127 downto 7120 ) when CO1024_dly2 = "0010000101" else 
X_RD_DATA( 7119 downto 7112 ) when CO1024_dly2 = "0010000110" else 
X_RD_DATA( 7111 downto 7104 ) when CO1024_dly2 = "0010000111" else 
X_RD_DATA( 7103 downto 7096 ) when CO1024_dly2 = "0010001000" else 
X_RD_DATA( 7095 downto 7088 ) when CO1024_dly2 = "0010001001" else 
X_RD_DATA( 7087 downto 7080 ) when CO1024_dly2 = "0010001010" else 
X_RD_DATA( 7079 downto 7072 ) when CO1024_dly2 = "0010001011" else 
X_RD_DATA( 7071 downto 7064 ) when CO1024_dly2 = "0010001100" else 
X_RD_DATA( 7063 downto 7056 ) when CO1024_dly2 = "0010001101" else 
X_RD_DATA( 7055 downto 7048 ) when CO1024_dly2 = "0010001110" else 
X_RD_DATA( 7047 downto 7040 ) when CO1024_dly2 = "0010001111" else 
X_RD_DATA( 7039 downto 7032 ) when CO1024_dly2 = "0010010000" else 
X_RD_DATA( 7031 downto 7024 ) when CO1024_dly2 = "0010010001" else 
X_RD_DATA( 7023 downto 7016 ) when CO1024_dly2 = "0010010010" else 
X_RD_DATA( 7015 downto 7008 ) when CO1024_dly2 = "0010010011" else 
X_RD_DATA( 7007 downto 7000 ) when CO1024_dly2 = "0010010100" else 
X_RD_DATA( 6999 downto 6992 ) when CO1024_dly2 = "0010010101" else 
X_RD_DATA( 6991 downto 6984 ) when CO1024_dly2 = "0010010110" else 
X_RD_DATA( 6983 downto 6976 ) when CO1024_dly2 = "0010010111" else 
X_RD_DATA( 6975 downto 6968 ) when CO1024_dly2 = "0010011000" else 
X_RD_DATA( 6967 downto 6960 ) when CO1024_dly2 = "0010011001" else 
X_RD_DATA( 6959 downto 6952 ) when CO1024_dly2 = "0010011010" else 
X_RD_DATA( 6951 downto 6944 ) when CO1024_dly2 = "0010011011" else 
X_RD_DATA( 6943 downto 6936 ) when CO1024_dly2 = "0010011100" else 
X_RD_DATA( 6935 downto 6928 ) when CO1024_dly2 = "0010011101" else 
X_RD_DATA( 6927 downto 6920 ) when CO1024_dly2 = "0010011110" else 
X_RD_DATA( 6919 downto 6912 ) when CO1024_dly2 = "0010011111" else 
X_RD_DATA( 6911 downto 6904 ) when CO1024_dly2 = "0010100000" else 
X_RD_DATA( 6903 downto 6896 ) when CO1024_dly2 = "0010100001" else 
X_RD_DATA( 6895 downto 6888 ) when CO1024_dly2 = "0010100010" else 
X_RD_DATA( 6887 downto 6880 ) when CO1024_dly2 = "0010100011" else 
X_RD_DATA( 6879 downto 6872 ) when CO1024_dly2 = "0010100100" else 
X_RD_DATA( 6871 downto 6864 ) when CO1024_dly2 = "0010100101" else 
X_RD_DATA( 6863 downto 6856 ) when CO1024_dly2 = "0010100110" else 
X_RD_DATA( 6855 downto 6848 ) when CO1024_dly2 = "0010100111" else 
X_RD_DATA( 6847 downto 6840 ) when CO1024_dly2 = "0010101000" else 
X_RD_DATA( 6839 downto 6832 ) when CO1024_dly2 = "0010101001" else 
X_RD_DATA( 6831 downto 6824 ) when CO1024_dly2 = "0010101010" else 
X_RD_DATA( 6823 downto 6816 ) when CO1024_dly2 = "0010101011" else 
X_RD_DATA( 6815 downto 6808 ) when CO1024_dly2 = "0010101100" else 
X_RD_DATA( 6807 downto 6800 ) when CO1024_dly2 = "0010101101" else 
X_RD_DATA( 6799 downto 6792 ) when CO1024_dly2 = "0010101110" else 
X_RD_DATA( 6791 downto 6784 ) when CO1024_dly2 = "0010101111" else 
X_RD_DATA( 6783 downto 6776 ) when CO1024_dly2 = "0010110000" else 
X_RD_DATA( 6775 downto 6768 ) when CO1024_dly2 = "0010110001" else 
X_RD_DATA( 6767 downto 6760 ) when CO1024_dly2 = "0010110010" else 
X_RD_DATA( 6759 downto 6752 ) when CO1024_dly2 = "0010110011" else 
X_RD_DATA( 6751 downto 6744 ) when CO1024_dly2 = "0010110100" else 
X_RD_DATA( 6743 downto 6736 ) when CO1024_dly2 = "0010110101" else 
X_RD_DATA( 6735 downto 6728 ) when CO1024_dly2 = "0010110110" else 
X_RD_DATA( 6727 downto 6720 ) when CO1024_dly2 = "0010110111" else 
X_RD_DATA( 6719 downto 6712 ) when CO1024_dly2 = "0010111000" else 
X_RD_DATA( 6711 downto 6704 ) when CO1024_dly2 = "0010111001" else 
X_RD_DATA( 6703 downto 6696 ) when CO1024_dly2 = "0010111010" else 
X_RD_DATA( 6695 downto 6688 ) when CO1024_dly2 = "0010111011" else 
X_RD_DATA( 6687 downto 6680 ) when CO1024_dly2 = "0010111100" else 
X_RD_DATA( 6679 downto 6672 ) when CO1024_dly2 = "0010111101" else 
X_RD_DATA( 6671 downto 6664 ) when CO1024_dly2 = "0010111110" else 
X_RD_DATA( 6663 downto 6656 ) when CO1024_dly2 = "0010111111" else 
X_RD_DATA( 6655 downto 6648 ) when CO1024_dly2 = "0011000000" else 
X_RD_DATA( 6647 downto 6640 ) when CO1024_dly2 = "0011000001" else 
X_RD_DATA( 6639 downto 6632 ) when CO1024_dly2 = "0011000010" else 
X_RD_DATA( 6631 downto 6624 ) when CO1024_dly2 = "0011000011" else 
X_RD_DATA( 6623 downto 6616 ) when CO1024_dly2 = "0011000100" else 
X_RD_DATA( 6615 downto 6608 ) when CO1024_dly2 = "0011000101" else 
X_RD_DATA( 6607 downto 6600 ) when CO1024_dly2 = "0011000110" else 
X_RD_DATA( 6599 downto 6592 ) when CO1024_dly2 = "0011000111" else 
X_RD_DATA( 6591 downto 6584 ) when CO1024_dly2 = "0011001000" else 
X_RD_DATA( 6583 downto 6576 ) when CO1024_dly2 = "0011001001" else 
X_RD_DATA( 6575 downto 6568 ) when CO1024_dly2 = "0011001010" else 
X_RD_DATA( 6567 downto 6560 ) when CO1024_dly2 = "0011001011" else 
X_RD_DATA( 6559 downto 6552 ) when CO1024_dly2 = "0011001100" else 
X_RD_DATA( 6551 downto 6544 ) when CO1024_dly2 = "0011001101" else 
X_RD_DATA( 6543 downto 6536 ) when CO1024_dly2 = "0011001110" else 
X_RD_DATA( 6535 downto 6528 ) when CO1024_dly2 = "0011001111" else 
X_RD_DATA( 6527 downto 6520 ) when CO1024_dly2 = "0011010000" else 
X_RD_DATA( 6519 downto 6512 ) when CO1024_dly2 = "0011010001" else 
X_RD_DATA( 6511 downto 6504 ) when CO1024_dly2 = "0011010010" else 
X_RD_DATA( 6503 downto 6496 ) when CO1024_dly2 = "0011010011" else 
X_RD_DATA( 6495 downto 6488 ) when CO1024_dly2 = "0011010100" else 
X_RD_DATA( 6487 downto 6480 ) when CO1024_dly2 = "0011010101" else 
X_RD_DATA( 6479 downto 6472 ) when CO1024_dly2 = "0011010110" else 
X_RD_DATA( 6471 downto 6464 ) when CO1024_dly2 = "0011010111" else 
X_RD_DATA( 6463 downto 6456 ) when CO1024_dly2 = "0011011000" else 
X_RD_DATA( 6455 downto 6448 ) when CO1024_dly2 = "0011011001" else 
X_RD_DATA( 6447 downto 6440 ) when CO1024_dly2 = "0011011010" else 
X_RD_DATA( 6439 downto 6432 ) when CO1024_dly2 = "0011011011" else 
X_RD_DATA( 6431 downto 6424 ) when CO1024_dly2 = "0011011100" else 
X_RD_DATA( 6423 downto 6416 ) when CO1024_dly2 = "0011011101" else 
X_RD_DATA( 6415 downto 6408 ) when CO1024_dly2 = "0011011110" else 
X_RD_DATA( 6407 downto 6400 ) when CO1024_dly2 = "0011011111" else 
X_RD_DATA( 6399 downto 6392 ) when CO1024_dly2 = "0011100000" else 
X_RD_DATA( 6391 downto 6384 ) when CO1024_dly2 = "0011100001" else 
X_RD_DATA( 6383 downto 6376 ) when CO1024_dly2 = "0011100010" else 
X_RD_DATA( 6375 downto 6368 ) when CO1024_dly2 = "0011100011" else 
X_RD_DATA( 6367 downto 6360 ) when CO1024_dly2 = "0011100100" else 
X_RD_DATA( 6359 downto 6352 ) when CO1024_dly2 = "0011100101" else 
X_RD_DATA( 6351 downto 6344 ) when CO1024_dly2 = "0011100110" else 
X_RD_DATA( 6343 downto 6336 ) when CO1024_dly2 = "0011100111" else 
X_RD_DATA( 6335 downto 6328 ) when CO1024_dly2 = "0011101000" else 
X_RD_DATA( 6327 downto 6320 ) when CO1024_dly2 = "0011101001" else 
X_RD_DATA( 6319 downto 6312 ) when CO1024_dly2 = "0011101010" else 
X_RD_DATA( 6311 downto 6304 ) when CO1024_dly2 = "0011101011" else 
X_RD_DATA( 6303 downto 6296 ) when CO1024_dly2 = "0011101100" else 
X_RD_DATA( 6295 downto 6288 ) when CO1024_dly2 = "0011101101" else 
X_RD_DATA( 6287 downto 6280 ) when CO1024_dly2 = "0011101110" else 
X_RD_DATA( 6279 downto 6272 ) when CO1024_dly2 = "0011101111" else 
X_RD_DATA( 6271 downto 6264 ) when CO1024_dly2 = "0011110000" else 
X_RD_DATA( 6263 downto 6256 ) when CO1024_dly2 = "0011110001" else 
X_RD_DATA( 6255 downto 6248 ) when CO1024_dly2 = "0011110010" else 
X_RD_DATA( 6247 downto 6240 ) when CO1024_dly2 = "0011110011" else 
X_RD_DATA( 6239 downto 6232 ) when CO1024_dly2 = "0011110100" else 
X_RD_DATA( 6231 downto 6224 ) when CO1024_dly2 = "0011110101" else 
X_RD_DATA( 6223 downto 6216 ) when CO1024_dly2 = "0011110110" else 
X_RD_DATA( 6215 downto 6208 ) when CO1024_dly2 = "0011110111" else 
X_RD_DATA( 6207 downto 6200 ) when CO1024_dly2 = "0011111000" else 
X_RD_DATA( 6199 downto 6192 ) when CO1024_dly2 = "0011111001" else 
X_RD_DATA( 6191 downto 6184 ) when CO1024_dly2 = "0011111010" else 
X_RD_DATA( 6183 downto 6176 ) when CO1024_dly2 = "0011111011" else 
X_RD_DATA( 6175 downto 6168 ) when CO1024_dly2 = "0011111100" else 
X_RD_DATA( 6167 downto 6160 ) when CO1024_dly2 = "0011111101" else 
X_RD_DATA( 6159 downto 6152 ) when CO1024_dly2 = "0011111110" else 
X_RD_DATA( 6151 downto 6144 ) when CO1024_dly2 = "0011111111" else 
X_RD_DATA( 6143 downto 6136 ) when CO1024_dly2 = "0100000000" else 
X_RD_DATA( 6135 downto 6128 ) when CO1024_dly2 = "0100000001" else 
X_RD_DATA( 6127 downto 6120 ) when CO1024_dly2 = "0100000010" else 
X_RD_DATA( 6119 downto 6112 ) when CO1024_dly2 = "0100000011" else 
X_RD_DATA( 6111 downto 6104 ) when CO1024_dly2 = "0100000100" else 
X_RD_DATA( 6103 downto 6096 ) when CO1024_dly2 = "0100000101" else 
X_RD_DATA( 6095 downto 6088 ) when CO1024_dly2 = "0100000110" else 
X_RD_DATA( 6087 downto 6080 ) when CO1024_dly2 = "0100000111" else 
X_RD_DATA( 6079 downto 6072 ) when CO1024_dly2 = "0100001000" else 
X_RD_DATA( 6071 downto 6064 ) when CO1024_dly2 = "0100001001" else 
X_RD_DATA( 6063 downto 6056 ) when CO1024_dly2 = "0100001010" else 
X_RD_DATA( 6055 downto 6048 ) when CO1024_dly2 = "0100001011" else 
X_RD_DATA( 6047 downto 6040 ) when CO1024_dly2 = "0100001100" else 
X_RD_DATA( 6039 downto 6032 ) when CO1024_dly2 = "0100001101" else 
X_RD_DATA( 6031 downto 6024 ) when CO1024_dly2 = "0100001110" else 
X_RD_DATA( 6023 downto 6016 ) when CO1024_dly2 = "0100001111" else 
X_RD_DATA( 6015 downto 6008 ) when CO1024_dly2 = "0100010000" else 
X_RD_DATA( 6007 downto 6000 ) when CO1024_dly2 = "0100010001" else 
X_RD_DATA( 5999 downto 5992 ) when CO1024_dly2 = "0100010010" else 
X_RD_DATA( 5991 downto 5984 ) when CO1024_dly2 = "0100010011" else 
X_RD_DATA( 5983 downto 5976 ) when CO1024_dly2 = "0100010100" else 
X_RD_DATA( 5975 downto 5968 ) when CO1024_dly2 = "0100010101" else 
X_RD_DATA( 5967 downto 5960 ) when CO1024_dly2 = "0100010110" else 
X_RD_DATA( 5959 downto 5952 ) when CO1024_dly2 = "0100010111" else 
X_RD_DATA( 5951 downto 5944 ) when CO1024_dly2 = "0100011000" else 
X_RD_DATA( 5943 downto 5936 ) when CO1024_dly2 = "0100011001" else 
X_RD_DATA( 5935 downto 5928 ) when CO1024_dly2 = "0100011010" else 
X_RD_DATA( 5927 downto 5920 ) when CO1024_dly2 = "0100011011" else 
X_RD_DATA( 5919 downto 5912 ) when CO1024_dly2 = "0100011100" else 
X_RD_DATA( 5911 downto 5904 ) when CO1024_dly2 = "0100011101" else 
X_RD_DATA( 5903 downto 5896 ) when CO1024_dly2 = "0100011110" else 
X_RD_DATA( 5895 downto 5888 ) when CO1024_dly2 = "0100011111" else 
X_RD_DATA( 5887 downto 5880 ) when CO1024_dly2 = "0100100000" else 
X_RD_DATA( 5879 downto 5872 ) when CO1024_dly2 = "0100100001" else 
X_RD_DATA( 5871 downto 5864 ) when CO1024_dly2 = "0100100010" else 
X_RD_DATA( 5863 downto 5856 ) when CO1024_dly2 = "0100100011" else 
X_RD_DATA( 5855 downto 5848 ) when CO1024_dly2 = "0100100100" else 
X_RD_DATA( 5847 downto 5840 ) when CO1024_dly2 = "0100100101" else 
X_RD_DATA( 5839 downto 5832 ) when CO1024_dly2 = "0100100110" else 
X_RD_DATA( 5831 downto 5824 ) when CO1024_dly2 = "0100100111" else 
X_RD_DATA( 5823 downto 5816 ) when CO1024_dly2 = "0100101000" else 
X_RD_DATA( 5815 downto 5808 ) when CO1024_dly2 = "0100101001" else 
X_RD_DATA( 5807 downto 5800 ) when CO1024_dly2 = "0100101010" else 
X_RD_DATA( 5799 downto 5792 ) when CO1024_dly2 = "0100101011" else 
X_RD_DATA( 5791 downto 5784 ) when CO1024_dly2 = "0100101100" else 
X_RD_DATA( 5783 downto 5776 ) when CO1024_dly2 = "0100101101" else 
X_RD_DATA( 5775 downto 5768 ) when CO1024_dly2 = "0100101110" else 
X_RD_DATA( 5767 downto 5760 ) when CO1024_dly2 = "0100101111" else 
X_RD_DATA( 5759 downto 5752 ) when CO1024_dly2 = "0100110000" else 
X_RD_DATA( 5751 downto 5744 ) when CO1024_dly2 = "0100110001" else 
X_RD_DATA( 5743 downto 5736 ) when CO1024_dly2 = "0100110010" else 
X_RD_DATA( 5735 downto 5728 ) when CO1024_dly2 = "0100110011" else 
X_RD_DATA( 5727 downto 5720 ) when CO1024_dly2 = "0100110100" else 
X_RD_DATA( 5719 downto 5712 ) when CO1024_dly2 = "0100110101" else 
X_RD_DATA( 5711 downto 5704 ) when CO1024_dly2 = "0100110110" else 
X_RD_DATA( 5703 downto 5696 ) when CO1024_dly2 = "0100110111" else 
X_RD_DATA( 5695 downto 5688 ) when CO1024_dly2 = "0100111000" else 
X_RD_DATA( 5687 downto 5680 ) when CO1024_dly2 = "0100111001" else 
X_RD_DATA( 5679 downto 5672 ) when CO1024_dly2 = "0100111010" else 
X_RD_DATA( 5671 downto 5664 ) when CO1024_dly2 = "0100111011" else 
X_RD_DATA( 5663 downto 5656 ) when CO1024_dly2 = "0100111100" else 
X_RD_DATA( 5655 downto 5648 ) when CO1024_dly2 = "0100111101" else 
X_RD_DATA( 5647 downto 5640 ) when CO1024_dly2 = "0100111110" else 
X_RD_DATA( 5639 downto 5632 ) when CO1024_dly2 = "0100111111" else 
X_RD_DATA( 5631 downto 5624 ) when CO1024_dly2 = "0101000000" else 
X_RD_DATA( 5623 downto 5616 ) when CO1024_dly2 = "0101000001" else 
X_RD_DATA( 5615 downto 5608 ) when CO1024_dly2 = "0101000010" else 
X_RD_DATA( 5607 downto 5600 ) when CO1024_dly2 = "0101000011" else 
X_RD_DATA( 5599 downto 5592 ) when CO1024_dly2 = "0101000100" else 
X_RD_DATA( 5591 downto 5584 ) when CO1024_dly2 = "0101000101" else 
X_RD_DATA( 5583 downto 5576 ) when CO1024_dly2 = "0101000110" else 
X_RD_DATA( 5575 downto 5568 ) when CO1024_dly2 = "0101000111" else 
X_RD_DATA( 5567 downto 5560 ) when CO1024_dly2 = "0101001000" else 
X_RD_DATA( 5559 downto 5552 ) when CO1024_dly2 = "0101001001" else 
X_RD_DATA( 5551 downto 5544 ) when CO1024_dly2 = "0101001010" else 
X_RD_DATA( 5543 downto 5536 ) when CO1024_dly2 = "0101001011" else 
X_RD_DATA( 5535 downto 5528 ) when CO1024_dly2 = "0101001100" else 
X_RD_DATA( 5527 downto 5520 ) when CO1024_dly2 = "0101001101" else 
X_RD_DATA( 5519 downto 5512 ) when CO1024_dly2 = "0101001110" else 
X_RD_DATA( 5511 downto 5504 ) when CO1024_dly2 = "0101001111" else 
X_RD_DATA( 5503 downto 5496 ) when CO1024_dly2 = "0101010000" else 
X_RD_DATA( 5495 downto 5488 ) when CO1024_dly2 = "0101010001" else 
X_RD_DATA( 5487 downto 5480 ) when CO1024_dly2 = "0101010010" else 
X_RD_DATA( 5479 downto 5472 ) when CO1024_dly2 = "0101010011" else 
X_RD_DATA( 5471 downto 5464 ) when CO1024_dly2 = "0101010100" else 
X_RD_DATA( 5463 downto 5456 ) when CO1024_dly2 = "0101010101" else 
X_RD_DATA( 5455 downto 5448 ) when CO1024_dly2 = "0101010110" else 
X_RD_DATA( 5447 downto 5440 ) when CO1024_dly2 = "0101010111" else 
X_RD_DATA( 5439 downto 5432 ) when CO1024_dly2 = "0101011000" else 
X_RD_DATA( 5431 downto 5424 ) when CO1024_dly2 = "0101011001" else 
X_RD_DATA( 5423 downto 5416 ) when CO1024_dly2 = "0101011010" else 
X_RD_DATA( 5415 downto 5408 ) when CO1024_dly2 = "0101011011" else 
X_RD_DATA( 5407 downto 5400 ) when CO1024_dly2 = "0101011100" else 
X_RD_DATA( 5399 downto 5392 ) when CO1024_dly2 = "0101011101" else 
X_RD_DATA( 5391 downto 5384 ) when CO1024_dly2 = "0101011110" else 
X_RD_DATA( 5383 downto 5376 ) when CO1024_dly2 = "0101011111" else 
X_RD_DATA( 5375 downto 5368 ) when CO1024_dly2 = "0101100000" else 
X_RD_DATA( 5367 downto 5360 ) when CO1024_dly2 = "0101100001" else 
X_RD_DATA( 5359 downto 5352 ) when CO1024_dly2 = "0101100010" else 
X_RD_DATA( 5351 downto 5344 ) when CO1024_dly2 = "0101100011" else 
X_RD_DATA( 5343 downto 5336 ) when CO1024_dly2 = "0101100100" else 
X_RD_DATA( 5335 downto 5328 ) when CO1024_dly2 = "0101100101" else 
X_RD_DATA( 5327 downto 5320 ) when CO1024_dly2 = "0101100110" else 
X_RD_DATA( 5319 downto 5312 ) when CO1024_dly2 = "0101100111" else 
X_RD_DATA( 5311 downto 5304 ) when CO1024_dly2 = "0101101000" else 
X_RD_DATA( 5303 downto 5296 ) when CO1024_dly2 = "0101101001" else 
X_RD_DATA( 5295 downto 5288 ) when CO1024_dly2 = "0101101010" else 
X_RD_DATA( 5287 downto 5280 ) when CO1024_dly2 = "0101101011" else 
X_RD_DATA( 5279 downto 5272 ) when CO1024_dly2 = "0101101100" else 
X_RD_DATA( 5271 downto 5264 ) when CO1024_dly2 = "0101101101" else 
X_RD_DATA( 5263 downto 5256 ) when CO1024_dly2 = "0101101110" else 
X_RD_DATA( 5255 downto 5248 ) when CO1024_dly2 = "0101101111" else 
X_RD_DATA( 5247 downto 5240 ) when CO1024_dly2 = "0101110000" else 
X_RD_DATA( 5239 downto 5232 ) when CO1024_dly2 = "0101110001" else 
X_RD_DATA( 5231 downto 5224 ) when CO1024_dly2 = "0101110010" else 
X_RD_DATA( 5223 downto 5216 ) when CO1024_dly2 = "0101110011" else 
X_RD_DATA( 5215 downto 5208 ) when CO1024_dly2 = "0101110100" else 
X_RD_DATA( 5207 downto 5200 ) when CO1024_dly2 = "0101110101" else 
X_RD_DATA( 5199 downto 5192 ) when CO1024_dly2 = "0101110110" else 
X_RD_DATA( 5191 downto 5184 ) when CO1024_dly2 = "0101110111" else 
X_RD_DATA( 5183 downto 5176 ) when CO1024_dly2 = "0101111000" else 
X_RD_DATA( 5175 downto 5168 ) when CO1024_dly2 = "0101111001" else 
X_RD_DATA( 5167 downto 5160 ) when CO1024_dly2 = "0101111010" else 
X_RD_DATA( 5159 downto 5152 ) when CO1024_dly2 = "0101111011" else 
X_RD_DATA( 5151 downto 5144 ) when CO1024_dly2 = "0101111100" else 
X_RD_DATA( 5143 downto 5136 ) when CO1024_dly2 = "0101111101" else 
X_RD_DATA( 5135 downto 5128 ) when CO1024_dly2 = "0101111110" else 
X_RD_DATA( 5127 downto 5120 ) when CO1024_dly2 = "0101111111" else 
X_RD_DATA( 5119 downto 5112 ) when CO1024_dly2 = "0110000000" else 
X_RD_DATA( 5111 downto 5104 ) when CO1024_dly2 = "0110000001" else 
X_RD_DATA( 5103 downto 5096 ) when CO1024_dly2 = "0110000010" else 
X_RD_DATA( 5095 downto 5088 ) when CO1024_dly2 = "0110000011" else 
X_RD_DATA( 5087 downto 5080 ) when CO1024_dly2 = "0110000100" else 
X_RD_DATA( 5079 downto 5072 ) when CO1024_dly2 = "0110000101" else 
X_RD_DATA( 5071 downto 5064 ) when CO1024_dly2 = "0110000110" else 
X_RD_DATA( 5063 downto 5056 ) when CO1024_dly2 = "0110000111" else 
X_RD_DATA( 5055 downto 5048 ) when CO1024_dly2 = "0110001000" else 
X_RD_DATA( 5047 downto 5040 ) when CO1024_dly2 = "0110001001" else 
X_RD_DATA( 5039 downto 5032 ) when CO1024_dly2 = "0110001010" else 
X_RD_DATA( 5031 downto 5024 ) when CO1024_dly2 = "0110001011" else 
X_RD_DATA( 5023 downto 5016 ) when CO1024_dly2 = "0110001100" else 
X_RD_DATA( 5015 downto 5008 ) when CO1024_dly2 = "0110001101" else 
X_RD_DATA( 5007 downto 5000 ) when CO1024_dly2 = "0110001110" else 
X_RD_DATA( 4999 downto 4992 ) when CO1024_dly2 = "0110001111" else 
X_RD_DATA( 4991 downto 4984 ) when CO1024_dly2 = "0110010000" else 
X_RD_DATA( 4983 downto 4976 ) when CO1024_dly2 = "0110010001" else 
X_RD_DATA( 4975 downto 4968 ) when CO1024_dly2 = "0110010010" else 
X_RD_DATA( 4967 downto 4960 ) when CO1024_dly2 = "0110010011" else 
X_RD_DATA( 4959 downto 4952 ) when CO1024_dly2 = "0110010100" else 
X_RD_DATA( 4951 downto 4944 ) when CO1024_dly2 = "0110010101" else 
X_RD_DATA( 4943 downto 4936 ) when CO1024_dly2 = "0110010110" else 
X_RD_DATA( 4935 downto 4928 ) when CO1024_dly2 = "0110010111" else 
X_RD_DATA( 4927 downto 4920 ) when CO1024_dly2 = "0110011000" else 
X_RD_DATA( 4919 downto 4912 ) when CO1024_dly2 = "0110011001" else 
X_RD_DATA( 4911 downto 4904 ) when CO1024_dly2 = "0110011010" else 
X_RD_DATA( 4903 downto 4896 ) when CO1024_dly2 = "0110011011" else 
X_RD_DATA( 4895 downto 4888 ) when CO1024_dly2 = "0110011100" else 
X_RD_DATA( 4887 downto 4880 ) when CO1024_dly2 = "0110011101" else 
X_RD_DATA( 4879 downto 4872 ) when CO1024_dly2 = "0110011110" else 
X_RD_DATA( 4871 downto 4864 ) when CO1024_dly2 = "0110011111" else 
X_RD_DATA( 4863 downto 4856 ) when CO1024_dly2 = "0110100000" else 
X_RD_DATA( 4855 downto 4848 ) when CO1024_dly2 = "0110100001" else 
X_RD_DATA( 4847 downto 4840 ) when CO1024_dly2 = "0110100010" else 
X_RD_DATA( 4839 downto 4832 ) when CO1024_dly2 = "0110100011" else 
X_RD_DATA( 4831 downto 4824 ) when CO1024_dly2 = "0110100100" else 
X_RD_DATA( 4823 downto 4816 ) when CO1024_dly2 = "0110100101" else 
X_RD_DATA( 4815 downto 4808 ) when CO1024_dly2 = "0110100110" else 
X_RD_DATA( 4807 downto 4800 ) when CO1024_dly2 = "0110100111" else 
X_RD_DATA( 4799 downto 4792 ) when CO1024_dly2 = "0110101000" else 
X_RD_DATA( 4791 downto 4784 ) when CO1024_dly2 = "0110101001" else 
X_RD_DATA( 4783 downto 4776 ) when CO1024_dly2 = "0110101010" else 
X_RD_DATA( 4775 downto 4768 ) when CO1024_dly2 = "0110101011" else 
X_RD_DATA( 4767 downto 4760 ) when CO1024_dly2 = "0110101100" else 
X_RD_DATA( 4759 downto 4752 ) when CO1024_dly2 = "0110101101" else 
X_RD_DATA( 4751 downto 4744 ) when CO1024_dly2 = "0110101110" else 
X_RD_DATA( 4743 downto 4736 ) when CO1024_dly2 = "0110101111" else 
X_RD_DATA( 4735 downto 4728 ) when CO1024_dly2 = "0110110000" else 
X_RD_DATA( 4727 downto 4720 ) when CO1024_dly2 = "0110110001" else 
X_RD_DATA( 4719 downto 4712 ) when CO1024_dly2 = "0110110010" else 
X_RD_DATA( 4711 downto 4704 ) when CO1024_dly2 = "0110110011" else 
X_RD_DATA( 4703 downto 4696 ) when CO1024_dly2 = "0110110100" else 
X_RD_DATA( 4695 downto 4688 ) when CO1024_dly2 = "0110110101" else 
X_RD_DATA( 4687 downto 4680 ) when CO1024_dly2 = "0110110110" else 
X_RD_DATA( 4679 downto 4672 ) when CO1024_dly2 = "0110110111" else 
X_RD_DATA( 4671 downto 4664 ) when CO1024_dly2 = "0110111000" else 
X_RD_DATA( 4663 downto 4656 ) when CO1024_dly2 = "0110111001" else 
X_RD_DATA( 4655 downto 4648 ) when CO1024_dly2 = "0110111010" else 
X_RD_DATA( 4647 downto 4640 ) when CO1024_dly2 = "0110111011" else 
X_RD_DATA( 4639 downto 4632 ) when CO1024_dly2 = "0110111100" else 
X_RD_DATA( 4631 downto 4624 ) when CO1024_dly2 = "0110111101" else 
X_RD_DATA( 4623 downto 4616 ) when CO1024_dly2 = "0110111110" else 
X_RD_DATA( 4615 downto 4608 ) when CO1024_dly2 = "0110111111" else 
X_RD_DATA( 4607 downto 4600 ) when CO1024_dly2 = "0111000000" else 
X_RD_DATA( 4599 downto 4592 ) when CO1024_dly2 = "0111000001" else 
X_RD_DATA( 4591 downto 4584 ) when CO1024_dly2 = "0111000010" else 
X_RD_DATA( 4583 downto 4576 ) when CO1024_dly2 = "0111000011" else 
X_RD_DATA( 4575 downto 4568 ) when CO1024_dly2 = "0111000100" else 
X_RD_DATA( 4567 downto 4560 ) when CO1024_dly2 = "0111000101" else 
X_RD_DATA( 4559 downto 4552 ) when CO1024_dly2 = "0111000110" else 
X_RD_DATA( 4551 downto 4544 ) when CO1024_dly2 = "0111000111" else 
X_RD_DATA( 4543 downto 4536 ) when CO1024_dly2 = "0111001000" else 
X_RD_DATA( 4535 downto 4528 ) when CO1024_dly2 = "0111001001" else 
X_RD_DATA( 4527 downto 4520 ) when CO1024_dly2 = "0111001010" else 
X_RD_DATA( 4519 downto 4512 ) when CO1024_dly2 = "0111001011" else 
X_RD_DATA( 4511 downto 4504 ) when CO1024_dly2 = "0111001100" else 
X_RD_DATA( 4503 downto 4496 ) when CO1024_dly2 = "0111001101" else 
X_RD_DATA( 4495 downto 4488 ) when CO1024_dly2 = "0111001110" else 
X_RD_DATA( 4487 downto 4480 ) when CO1024_dly2 = "0111001111" else 
X_RD_DATA( 4479 downto 4472 ) when CO1024_dly2 = "0111010000" else 
X_RD_DATA( 4471 downto 4464 ) when CO1024_dly2 = "0111010001" else 
X_RD_DATA( 4463 downto 4456 ) when CO1024_dly2 = "0111010010" else 
X_RD_DATA( 4455 downto 4448 ) when CO1024_dly2 = "0111010011" else 
X_RD_DATA( 4447 downto 4440 ) when CO1024_dly2 = "0111010100" else 
X_RD_DATA( 4439 downto 4432 ) when CO1024_dly2 = "0111010101" else 
X_RD_DATA( 4431 downto 4424 ) when CO1024_dly2 = "0111010110" else 
X_RD_DATA( 4423 downto 4416 ) when CO1024_dly2 = "0111010111" else 
X_RD_DATA( 4415 downto 4408 ) when CO1024_dly2 = "0111011000" else 
X_RD_DATA( 4407 downto 4400 ) when CO1024_dly2 = "0111011001" else 
X_RD_DATA( 4399 downto 4392 ) when CO1024_dly2 = "0111011010" else 
X_RD_DATA( 4391 downto 4384 ) when CO1024_dly2 = "0111011011" else 
X_RD_DATA( 4383 downto 4376 ) when CO1024_dly2 = "0111011100" else 
X_RD_DATA( 4375 downto 4368 ) when CO1024_dly2 = "0111011101" else 
X_RD_DATA( 4367 downto 4360 ) when CO1024_dly2 = "0111011110" else 
X_RD_DATA( 4359 downto 4352 ) when CO1024_dly2 = "0111011111" else 
X_RD_DATA( 4351 downto 4344 ) when CO1024_dly2 = "0111100000" else 
X_RD_DATA( 4343 downto 4336 ) when CO1024_dly2 = "0111100001" else 
X_RD_DATA( 4335 downto 4328 ) when CO1024_dly2 = "0111100010" else 
X_RD_DATA( 4327 downto 4320 ) when CO1024_dly2 = "0111100011" else 
X_RD_DATA( 4319 downto 4312 ) when CO1024_dly2 = "0111100100" else 
X_RD_DATA( 4311 downto 4304 ) when CO1024_dly2 = "0111100101" else 
X_RD_DATA( 4303 downto 4296 ) when CO1024_dly2 = "0111100110" else 
X_RD_DATA( 4295 downto 4288 ) when CO1024_dly2 = "0111100111" else 
X_RD_DATA( 4287 downto 4280 ) when CO1024_dly2 = "0111101000" else 
X_RD_DATA( 4279 downto 4272 ) when CO1024_dly2 = "0111101001" else 
X_RD_DATA( 4271 downto 4264 ) when CO1024_dly2 = "0111101010" else 
X_RD_DATA( 4263 downto 4256 ) when CO1024_dly2 = "0111101011" else 
X_RD_DATA( 4255 downto 4248 ) when CO1024_dly2 = "0111101100" else 
X_RD_DATA( 4247 downto 4240 ) when CO1024_dly2 = "0111101101" else 
X_RD_DATA( 4239 downto 4232 ) when CO1024_dly2 = "0111101110" else 
X_RD_DATA( 4231 downto 4224 ) when CO1024_dly2 = "0111101111" else 
X_RD_DATA( 4223 downto 4216 ) when CO1024_dly2 = "0111110000" else 
X_RD_DATA( 4215 downto 4208 ) when CO1024_dly2 = "0111110001" else 
X_RD_DATA( 4207 downto 4200 ) when CO1024_dly2 = "0111110010" else 
X_RD_DATA( 4199 downto 4192 ) when CO1024_dly2 = "0111110011" else 
X_RD_DATA( 4191 downto 4184 ) when CO1024_dly2 = "0111110100" else 
X_RD_DATA( 4183 downto 4176 ) when CO1024_dly2 = "0111110101" else 
X_RD_DATA( 4175 downto 4168 ) when CO1024_dly2 = "0111110110" else 
X_RD_DATA( 4167 downto 4160 ) when CO1024_dly2 = "0111110111" else 
X_RD_DATA( 4159 downto 4152 ) when CO1024_dly2 = "0111111000" else 
X_RD_DATA( 4151 downto 4144 ) when CO1024_dly2 = "0111111001" else 
X_RD_DATA( 4143 downto 4136 ) when CO1024_dly2 = "0111111010" else 
X_RD_DATA( 4135 downto 4128 ) when CO1024_dly2 = "0111111011" else 
X_RD_DATA( 4127 downto 4120 ) when CO1024_dly2 = "0111111100" else 
X_RD_DATA( 4119 downto 4112 ) when CO1024_dly2 = "0111111101" else 
X_RD_DATA( 4111 downto 4104 ) when CO1024_dly2 = "0111111110" else 
X_RD_DATA( 4103 downto 4096 ) when CO1024_dly2 = "0111111111" else 
X_RD_DATA( 4095 downto 4088 ) when CO1024_dly2 = "1000000000" else 
X_RD_DATA( 4087 downto 4080 ) when CO1024_dly2 = "1000000001" else 
X_RD_DATA( 4079 downto 4072 ) when CO1024_dly2 = "1000000010" else 
X_RD_DATA( 4071 downto 4064 ) when CO1024_dly2 = "1000000011" else 
X_RD_DATA( 4063 downto 4056 ) when CO1024_dly2 = "1000000100" else 
X_RD_DATA( 4055 downto 4048 ) when CO1024_dly2 = "1000000101" else 
X_RD_DATA( 4047 downto 4040 ) when CO1024_dly2 = "1000000110" else 
X_RD_DATA( 4039 downto 4032 ) when CO1024_dly2 = "1000000111" else 
X_RD_DATA( 4031 downto 4024 ) when CO1024_dly2 = "1000001000" else 
X_RD_DATA( 4023 downto 4016 ) when CO1024_dly2 = "1000001001" else 
X_RD_DATA( 4015 downto 4008 ) when CO1024_dly2 = "1000001010" else 
X_RD_DATA( 4007 downto 4000 ) when CO1024_dly2 = "1000001011" else 
X_RD_DATA( 3999 downto 3992 ) when CO1024_dly2 = "1000001100" else 
X_RD_DATA( 3991 downto 3984 ) when CO1024_dly2 = "1000001101" else 
X_RD_DATA( 3983 downto 3976 ) when CO1024_dly2 = "1000001110" else 
X_RD_DATA( 3975 downto 3968 ) when CO1024_dly2 = "1000001111" else 
X_RD_DATA( 3967 downto 3960 ) when CO1024_dly2 = "1000010000" else 
X_RD_DATA( 3959 downto 3952 ) when CO1024_dly2 = "1000010001" else 
X_RD_DATA( 3951 downto 3944 ) when CO1024_dly2 = "1000010010" else 
X_RD_DATA( 3943 downto 3936 ) when CO1024_dly2 = "1000010011" else 
X_RD_DATA( 3935 downto 3928 ) when CO1024_dly2 = "1000010100" else 
X_RD_DATA( 3927 downto 3920 ) when CO1024_dly2 = "1000010101" else 
X_RD_DATA( 3919 downto 3912 ) when CO1024_dly2 = "1000010110" else 
X_RD_DATA( 3911 downto 3904 ) when CO1024_dly2 = "1000010111" else 
X_RD_DATA( 3903 downto 3896 ) when CO1024_dly2 = "1000011000" else 
X_RD_DATA( 3895 downto 3888 ) when CO1024_dly2 = "1000011001" else 
X_RD_DATA( 3887 downto 3880 ) when CO1024_dly2 = "1000011010" else 
X_RD_DATA( 3879 downto 3872 ) when CO1024_dly2 = "1000011011" else 
X_RD_DATA( 3871 downto 3864 ) when CO1024_dly2 = "1000011100" else 
X_RD_DATA( 3863 downto 3856 ) when CO1024_dly2 = "1000011101" else 
X_RD_DATA( 3855 downto 3848 ) when CO1024_dly2 = "1000011110" else 
X_RD_DATA( 3847 downto 3840 ) when CO1024_dly2 = "1000011111" else 
X_RD_DATA( 3839 downto 3832 ) when CO1024_dly2 = "1000100000" else 
X_RD_DATA( 3831 downto 3824 ) when CO1024_dly2 = "1000100001" else 
X_RD_DATA( 3823 downto 3816 ) when CO1024_dly2 = "1000100010" else 
X_RD_DATA( 3815 downto 3808 ) when CO1024_dly2 = "1000100011" else 
X_RD_DATA( 3807 downto 3800 ) when CO1024_dly2 = "1000100100" else 
X_RD_DATA( 3799 downto 3792 ) when CO1024_dly2 = "1000100101" else 
X_RD_DATA( 3791 downto 3784 ) when CO1024_dly2 = "1000100110" else 
X_RD_DATA( 3783 downto 3776 ) when CO1024_dly2 = "1000100111" else 
X_RD_DATA( 3775 downto 3768 ) when CO1024_dly2 = "1000101000" else 
X_RD_DATA( 3767 downto 3760 ) when CO1024_dly2 = "1000101001" else 
X_RD_DATA( 3759 downto 3752 ) when CO1024_dly2 = "1000101010" else 
X_RD_DATA( 3751 downto 3744 ) when CO1024_dly2 = "1000101011" else 
X_RD_DATA( 3743 downto 3736 ) when CO1024_dly2 = "1000101100" else 
X_RD_DATA( 3735 downto 3728 ) when CO1024_dly2 = "1000101101" else 
X_RD_DATA( 3727 downto 3720 ) when CO1024_dly2 = "1000101110" else 
X_RD_DATA( 3719 downto 3712 ) when CO1024_dly2 = "1000101111" else 
X_RD_DATA( 3711 downto 3704 ) when CO1024_dly2 = "1000110000" else 
X_RD_DATA( 3703 downto 3696 ) when CO1024_dly2 = "1000110001" else 
X_RD_DATA( 3695 downto 3688 ) when CO1024_dly2 = "1000110010" else 
X_RD_DATA( 3687 downto 3680 ) when CO1024_dly2 = "1000110011" else 
X_RD_DATA( 3679 downto 3672 ) when CO1024_dly2 = "1000110100" else 
X_RD_DATA( 3671 downto 3664 ) when CO1024_dly2 = "1000110101" else 
X_RD_DATA( 3663 downto 3656 ) when CO1024_dly2 = "1000110110" else 
X_RD_DATA( 3655 downto 3648 ) when CO1024_dly2 = "1000110111" else 
X_RD_DATA( 3647 downto 3640 ) when CO1024_dly2 = "1000111000" else 
X_RD_DATA( 3639 downto 3632 ) when CO1024_dly2 = "1000111001" else 
X_RD_DATA( 3631 downto 3624 ) when CO1024_dly2 = "1000111010" else 
X_RD_DATA( 3623 downto 3616 ) when CO1024_dly2 = "1000111011" else 
X_RD_DATA( 3615 downto 3608 ) when CO1024_dly2 = "1000111100" else 
X_RD_DATA( 3607 downto 3600 ) when CO1024_dly2 = "1000111101" else 
X_RD_DATA( 3599 downto 3592 ) when CO1024_dly2 = "1000111110" else 
X_RD_DATA( 3591 downto 3584 ) when CO1024_dly2 = "1000111111" else 
X_RD_DATA( 3583 downto 3576 ) when CO1024_dly2 = "1001000000" else 
X_RD_DATA( 3575 downto 3568 ) when CO1024_dly2 = "1001000001" else 
X_RD_DATA( 3567 downto 3560 ) when CO1024_dly2 = "1001000010" else 
X_RD_DATA( 3559 downto 3552 ) when CO1024_dly2 = "1001000011" else 
X_RD_DATA( 3551 downto 3544 ) when CO1024_dly2 = "1001000100" else 
X_RD_DATA( 3543 downto 3536 ) when CO1024_dly2 = "1001000101" else 
X_RD_DATA( 3535 downto 3528 ) when CO1024_dly2 = "1001000110" else 
X_RD_DATA( 3527 downto 3520 ) when CO1024_dly2 = "1001000111" else 
X_RD_DATA( 3519 downto 3512 ) when CO1024_dly2 = "1001001000" else 
X_RD_DATA( 3511 downto 3504 ) when CO1024_dly2 = "1001001001" else 
X_RD_DATA( 3503 downto 3496 ) when CO1024_dly2 = "1001001010" else 
X_RD_DATA( 3495 downto 3488 ) when CO1024_dly2 = "1001001011" else 
X_RD_DATA( 3487 downto 3480 ) when CO1024_dly2 = "1001001100" else 
X_RD_DATA( 3479 downto 3472 ) when CO1024_dly2 = "1001001101" else 
X_RD_DATA( 3471 downto 3464 ) when CO1024_dly2 = "1001001110" else 
X_RD_DATA( 3463 downto 3456 ) when CO1024_dly2 = "1001001111" else 
X_RD_DATA( 3455 downto 3448 ) when CO1024_dly2 = "1001010000" else 
X_RD_DATA( 3447 downto 3440 ) when CO1024_dly2 = "1001010001" else 
X_RD_DATA( 3439 downto 3432 ) when CO1024_dly2 = "1001010010" else 
X_RD_DATA( 3431 downto 3424 ) when CO1024_dly2 = "1001010011" else 
X_RD_DATA( 3423 downto 3416 ) when CO1024_dly2 = "1001010100" else 
X_RD_DATA( 3415 downto 3408 ) when CO1024_dly2 = "1001010101" else 
X_RD_DATA( 3407 downto 3400 ) when CO1024_dly2 = "1001010110" else 
X_RD_DATA( 3399 downto 3392 ) when CO1024_dly2 = "1001010111" else 
X_RD_DATA( 3391 downto 3384 ) when CO1024_dly2 = "1001011000" else 
X_RD_DATA( 3383 downto 3376 ) when CO1024_dly2 = "1001011001" else 
X_RD_DATA( 3375 downto 3368 ) when CO1024_dly2 = "1001011010" else 
X_RD_DATA( 3367 downto 3360 ) when CO1024_dly2 = "1001011011" else 
X_RD_DATA( 3359 downto 3352 ) when CO1024_dly2 = "1001011100" else 
X_RD_DATA( 3351 downto 3344 ) when CO1024_dly2 = "1001011101" else 
X_RD_DATA( 3343 downto 3336 ) when CO1024_dly2 = "1001011110" else 
X_RD_DATA( 3335 downto 3328 ) when CO1024_dly2 = "1001011111" else 
X_RD_DATA( 3327 downto 3320 ) when CO1024_dly2 = "1001100000" else 
X_RD_DATA( 3319 downto 3312 ) when CO1024_dly2 = "1001100001" else 
X_RD_DATA( 3311 downto 3304 ) when CO1024_dly2 = "1001100010" else 
X_RD_DATA( 3303 downto 3296 ) when CO1024_dly2 = "1001100011" else 
X_RD_DATA( 3295 downto 3288 ) when CO1024_dly2 = "1001100100" else 
X_RD_DATA( 3287 downto 3280 ) when CO1024_dly2 = "1001100101" else 
X_RD_DATA( 3279 downto 3272 ) when CO1024_dly2 = "1001100110" else 
X_RD_DATA( 3271 downto 3264 ) when CO1024_dly2 = "1001100111" else 
X_RD_DATA( 3263 downto 3256 ) when CO1024_dly2 = "1001101000" else 
X_RD_DATA( 3255 downto 3248 ) when CO1024_dly2 = "1001101001" else 
X_RD_DATA( 3247 downto 3240 ) when CO1024_dly2 = "1001101010" else 
X_RD_DATA( 3239 downto 3232 ) when CO1024_dly2 = "1001101011" else 
X_RD_DATA( 3231 downto 3224 ) when CO1024_dly2 = "1001101100" else 
X_RD_DATA( 3223 downto 3216 ) when CO1024_dly2 = "1001101101" else 
X_RD_DATA( 3215 downto 3208 ) when CO1024_dly2 = "1001101110" else 
X_RD_DATA( 3207 downto 3200 ) when CO1024_dly2 = "1001101111" else 
X_RD_DATA( 3199 downto 3192 ) when CO1024_dly2 = "1001110000" else 
X_RD_DATA( 3191 downto 3184 ) when CO1024_dly2 = "1001110001" else 
X_RD_DATA( 3183 downto 3176 ) when CO1024_dly2 = "1001110010" else 
X_RD_DATA( 3175 downto 3168 ) when CO1024_dly2 = "1001110011" else 
X_RD_DATA( 3167 downto 3160 ) when CO1024_dly2 = "1001110100" else 
X_RD_DATA( 3159 downto 3152 ) when CO1024_dly2 = "1001110101" else 
X_RD_DATA( 3151 downto 3144 ) when CO1024_dly2 = "1001110110" else 
X_RD_DATA( 3143 downto 3136 ) when CO1024_dly2 = "1001110111" else 
X_RD_DATA( 3135 downto 3128 ) when CO1024_dly2 = "1001111000" else 
X_RD_DATA( 3127 downto 3120 ) when CO1024_dly2 = "1001111001" else 
X_RD_DATA( 3119 downto 3112 ) when CO1024_dly2 = "1001111010" else 
X_RD_DATA( 3111 downto 3104 ) when CO1024_dly2 = "1001111011" else 
X_RD_DATA( 3103 downto 3096 ) when CO1024_dly2 = "1001111100" else 
X_RD_DATA( 3095 downto 3088 ) when CO1024_dly2 = "1001111101" else 
X_RD_DATA( 3087 downto 3080 ) when CO1024_dly2 = "1001111110" else 
X_RD_DATA( 3079 downto 3072 ) when CO1024_dly2 = "1001111111" else 
X_RD_DATA( 3071 downto 3064 ) when CO1024_dly2 = "1010000000" else 
X_RD_DATA( 3063 downto 3056 ) when CO1024_dly2 = "1010000001" else 
X_RD_DATA( 3055 downto 3048 ) when CO1024_dly2 = "1010000010" else 
X_RD_DATA( 3047 downto 3040 ) when CO1024_dly2 = "1010000011" else 
X_RD_DATA( 3039 downto 3032 ) when CO1024_dly2 = "1010000100" else 
X_RD_DATA( 3031 downto 3024 ) when CO1024_dly2 = "1010000101" else 
X_RD_DATA( 3023 downto 3016 ) when CO1024_dly2 = "1010000110" else 
X_RD_DATA( 3015 downto 3008 ) when CO1024_dly2 = "1010000111" else 
X_RD_DATA( 3007 downto 3000 ) when CO1024_dly2 = "1010001000" else 
X_RD_DATA( 2999 downto 2992 ) when CO1024_dly2 = "1010001001" else 
X_RD_DATA( 2991 downto 2984 ) when CO1024_dly2 = "1010001010" else 
X_RD_DATA( 2983 downto 2976 ) when CO1024_dly2 = "1010001011" else 
X_RD_DATA( 2975 downto 2968 ) when CO1024_dly2 = "1010001100" else 
X_RD_DATA( 2967 downto 2960 ) when CO1024_dly2 = "1010001101" else 
X_RD_DATA( 2959 downto 2952 ) when CO1024_dly2 = "1010001110" else 
X_RD_DATA( 2951 downto 2944 ) when CO1024_dly2 = "1010001111" else 
X_RD_DATA( 2943 downto 2936 ) when CO1024_dly2 = "1010010000" else 
X_RD_DATA( 2935 downto 2928 ) when CO1024_dly2 = "1010010001" else 
X_RD_DATA( 2927 downto 2920 ) when CO1024_dly2 = "1010010010" else 
X_RD_DATA( 2919 downto 2912 ) when CO1024_dly2 = "1010010011" else 
X_RD_DATA( 2911 downto 2904 ) when CO1024_dly2 = "1010010100" else 
X_RD_DATA( 2903 downto 2896 ) when CO1024_dly2 = "1010010101" else 
X_RD_DATA( 2895 downto 2888 ) when CO1024_dly2 = "1010010110" else 
X_RD_DATA( 2887 downto 2880 ) when CO1024_dly2 = "1010010111" else 
X_RD_DATA( 2879 downto 2872 ) when CO1024_dly2 = "1010011000" else 
X_RD_DATA( 2871 downto 2864 ) when CO1024_dly2 = "1010011001" else 
X_RD_DATA( 2863 downto 2856 ) when CO1024_dly2 = "1010011010" else 
X_RD_DATA( 2855 downto 2848 ) when CO1024_dly2 = "1010011011" else 
X_RD_DATA( 2847 downto 2840 ) when CO1024_dly2 = "1010011100" else 
X_RD_DATA( 2839 downto 2832 ) when CO1024_dly2 = "1010011101" else 
X_RD_DATA( 2831 downto 2824 ) when CO1024_dly2 = "1010011110" else 
X_RD_DATA( 2823 downto 2816 ) when CO1024_dly2 = "1010011111" else 
X_RD_DATA( 2815 downto 2808 ) when CO1024_dly2 = "1010100000" else 
X_RD_DATA( 2807 downto 2800 ) when CO1024_dly2 = "1010100001" else 
X_RD_DATA( 2799 downto 2792 ) when CO1024_dly2 = "1010100010" else 
X_RD_DATA( 2791 downto 2784 ) when CO1024_dly2 = "1010100011" else 
X_RD_DATA( 2783 downto 2776 ) when CO1024_dly2 = "1010100100" else 
X_RD_DATA( 2775 downto 2768 ) when CO1024_dly2 = "1010100101" else 
X_RD_DATA( 2767 downto 2760 ) when CO1024_dly2 = "1010100110" else 
X_RD_DATA( 2759 downto 2752 ) when CO1024_dly2 = "1010100111" else 
X_RD_DATA( 2751 downto 2744 ) when CO1024_dly2 = "1010101000" else 
X_RD_DATA( 2743 downto 2736 ) when CO1024_dly2 = "1010101001" else 
X_RD_DATA( 2735 downto 2728 ) when CO1024_dly2 = "1010101010" else 
X_RD_DATA( 2727 downto 2720 ) when CO1024_dly2 = "1010101011" else 
X_RD_DATA( 2719 downto 2712 ) when CO1024_dly2 = "1010101100" else 
X_RD_DATA( 2711 downto 2704 ) when CO1024_dly2 = "1010101101" else 
X_RD_DATA( 2703 downto 2696 ) when CO1024_dly2 = "1010101110" else 
X_RD_DATA( 2695 downto 2688 ) when CO1024_dly2 = "1010101111" else 
X_RD_DATA( 2687 downto 2680 ) when CO1024_dly2 = "1010110000" else 
X_RD_DATA( 2679 downto 2672 ) when CO1024_dly2 = "1010110001" else 
X_RD_DATA( 2671 downto 2664 ) when CO1024_dly2 = "1010110010" else 
X_RD_DATA( 2663 downto 2656 ) when CO1024_dly2 = "1010110011" else 
X_RD_DATA( 2655 downto 2648 ) when CO1024_dly2 = "1010110100" else 
X_RD_DATA( 2647 downto 2640 ) when CO1024_dly2 = "1010110101" else 
X_RD_DATA( 2639 downto 2632 ) when CO1024_dly2 = "1010110110" else 
X_RD_DATA( 2631 downto 2624 ) when CO1024_dly2 = "1010110111" else 
X_RD_DATA( 2623 downto 2616 ) when CO1024_dly2 = "1010111000" else 
X_RD_DATA( 2615 downto 2608 ) when CO1024_dly2 = "1010111001" else 
X_RD_DATA( 2607 downto 2600 ) when CO1024_dly2 = "1010111010" else 
X_RD_DATA( 2599 downto 2592 ) when CO1024_dly2 = "1010111011" else 
X_RD_DATA( 2591 downto 2584 ) when CO1024_dly2 = "1010111100" else 
X_RD_DATA( 2583 downto 2576 ) when CO1024_dly2 = "1010111101" else 
X_RD_DATA( 2575 downto 2568 ) when CO1024_dly2 = "1010111110" else 
X_RD_DATA( 2567 downto 2560 ) when CO1024_dly2 = "1010111111" else 
X_RD_DATA( 2559 downto 2552 ) when CO1024_dly2 = "1011000000" else 
X_RD_DATA( 2551 downto 2544 ) when CO1024_dly2 = "1011000001" else 
X_RD_DATA( 2543 downto 2536 ) when CO1024_dly2 = "1011000010" else 
X_RD_DATA( 2535 downto 2528 ) when CO1024_dly2 = "1011000011" else 
X_RD_DATA( 2527 downto 2520 ) when CO1024_dly2 = "1011000100" else 
X_RD_DATA( 2519 downto 2512 ) when CO1024_dly2 = "1011000101" else 
X_RD_DATA( 2511 downto 2504 ) when CO1024_dly2 = "1011000110" else 
X_RD_DATA( 2503 downto 2496 ) when CO1024_dly2 = "1011000111" else 
X_RD_DATA( 2495 downto 2488 ) when CO1024_dly2 = "1011001000" else 
X_RD_DATA( 2487 downto 2480 ) when CO1024_dly2 = "1011001001" else 
X_RD_DATA( 2479 downto 2472 ) when CO1024_dly2 = "1011001010" else 
X_RD_DATA( 2471 downto 2464 ) when CO1024_dly2 = "1011001011" else 
X_RD_DATA( 2463 downto 2456 ) when CO1024_dly2 = "1011001100" else 
X_RD_DATA( 2455 downto 2448 ) when CO1024_dly2 = "1011001101" else 
X_RD_DATA( 2447 downto 2440 ) when CO1024_dly2 = "1011001110" else 
X_RD_DATA( 2439 downto 2432 ) when CO1024_dly2 = "1011001111" else 
X_RD_DATA( 2431 downto 2424 ) when CO1024_dly2 = "1011010000" else 
X_RD_DATA( 2423 downto 2416 ) when CO1024_dly2 = "1011010001" else 
X_RD_DATA( 2415 downto 2408 ) when CO1024_dly2 = "1011010010" else 
X_RD_DATA( 2407 downto 2400 ) when CO1024_dly2 = "1011010011" else 
X_RD_DATA( 2399 downto 2392 ) when CO1024_dly2 = "1011010100" else 
X_RD_DATA( 2391 downto 2384 ) when CO1024_dly2 = "1011010101" else 
X_RD_DATA( 2383 downto 2376 ) when CO1024_dly2 = "1011010110" else 
X_RD_DATA( 2375 downto 2368 ) when CO1024_dly2 = "1011010111" else 
X_RD_DATA( 2367 downto 2360 ) when CO1024_dly2 = "1011011000" else 
X_RD_DATA( 2359 downto 2352 ) when CO1024_dly2 = "1011011001" else 
X_RD_DATA( 2351 downto 2344 ) when CO1024_dly2 = "1011011010" else 
X_RD_DATA( 2343 downto 2336 ) when CO1024_dly2 = "1011011011" else 
X_RD_DATA( 2335 downto 2328 ) when CO1024_dly2 = "1011011100" else 
X_RD_DATA( 2327 downto 2320 ) when CO1024_dly2 = "1011011101" else 
X_RD_DATA( 2319 downto 2312 ) when CO1024_dly2 = "1011011110" else 
X_RD_DATA( 2311 downto 2304 ) when CO1024_dly2 = "1011011111" else 
X_RD_DATA( 2303 downto 2296 ) when CO1024_dly2 = "1011100000" else 
X_RD_DATA( 2295 downto 2288 ) when CO1024_dly2 = "1011100001" else 
X_RD_DATA( 2287 downto 2280 ) when CO1024_dly2 = "1011100010" else 
X_RD_DATA( 2279 downto 2272 ) when CO1024_dly2 = "1011100011" else 
X_RD_DATA( 2271 downto 2264 ) when CO1024_dly2 = "1011100100" else 
X_RD_DATA( 2263 downto 2256 ) when CO1024_dly2 = "1011100101" else 
X_RD_DATA( 2255 downto 2248 ) when CO1024_dly2 = "1011100110" else 
X_RD_DATA( 2247 downto 2240 ) when CO1024_dly2 = "1011100111" else 
X_RD_DATA( 2239 downto 2232 ) when CO1024_dly2 = "1011101000" else 
X_RD_DATA( 2231 downto 2224 ) when CO1024_dly2 = "1011101001" else 
X_RD_DATA( 2223 downto 2216 ) when CO1024_dly2 = "1011101010" else 
X_RD_DATA( 2215 downto 2208 ) when CO1024_dly2 = "1011101011" else 
X_RD_DATA( 2207 downto 2200 ) when CO1024_dly2 = "1011101100" else 
X_RD_DATA( 2199 downto 2192 ) when CO1024_dly2 = "1011101101" else 
X_RD_DATA( 2191 downto 2184 ) when CO1024_dly2 = "1011101110" else 
X_RD_DATA( 2183 downto 2176 ) when CO1024_dly2 = "1011101111" else 
X_RD_DATA( 2175 downto 2168 ) when CO1024_dly2 = "1011110000" else 
X_RD_DATA( 2167 downto 2160 ) when CO1024_dly2 = "1011110001" else 
X_RD_DATA( 2159 downto 2152 ) when CO1024_dly2 = "1011110010" else 
X_RD_DATA( 2151 downto 2144 ) when CO1024_dly2 = "1011110011" else 
X_RD_DATA( 2143 downto 2136 ) when CO1024_dly2 = "1011110100" else 
X_RD_DATA( 2135 downto 2128 ) when CO1024_dly2 = "1011110101" else 
X_RD_DATA( 2127 downto 2120 ) when CO1024_dly2 = "1011110110" else 
X_RD_DATA( 2119 downto 2112 ) when CO1024_dly2 = "1011110111" else 
X_RD_DATA( 2111 downto 2104 ) when CO1024_dly2 = "1011111000" else 
X_RD_DATA( 2103 downto 2096 ) when CO1024_dly2 = "1011111001" else 
X_RD_DATA( 2095 downto 2088 ) when CO1024_dly2 = "1011111010" else 
X_RD_DATA( 2087 downto 2080 ) when CO1024_dly2 = "1011111011" else 
X_RD_DATA( 2079 downto 2072 ) when CO1024_dly2 = "1011111100" else 
X_RD_DATA( 2071 downto 2064 ) when CO1024_dly2 = "1011111101" else 
X_RD_DATA( 2063 downto 2056 ) when CO1024_dly2 = "1011111110" else 
X_RD_DATA( 2055 downto 2048 ) when CO1024_dly2 = "1011111111" else 
X_RD_DATA( 2047 downto 2040 ) when CO1024_dly2 = "1100000000" else 
X_RD_DATA( 2039 downto 2032 ) when CO1024_dly2 = "1100000001" else 
X_RD_DATA( 2031 downto 2024 ) when CO1024_dly2 = "1100000010" else 
X_RD_DATA( 2023 downto 2016 ) when CO1024_dly2 = "1100000011" else 
X_RD_DATA( 2015 downto 2008 ) when CO1024_dly2 = "1100000100" else 
X_RD_DATA( 2007 downto 2000 ) when CO1024_dly2 = "1100000101" else 
X_RD_DATA( 1999 downto 1992 ) when CO1024_dly2 = "1100000110" else 
X_RD_DATA( 1991 downto 1984 ) when CO1024_dly2 = "1100000111" else 
X_RD_DATA( 1983 downto 1976 ) when CO1024_dly2 = "1100001000" else 
X_RD_DATA( 1975 downto 1968 ) when CO1024_dly2 = "1100001001" else 
X_RD_DATA( 1967 downto 1960 ) when CO1024_dly2 = "1100001010" else 
X_RD_DATA( 1959 downto 1952 ) when CO1024_dly2 = "1100001011" else 
X_RD_DATA( 1951 downto 1944 ) when CO1024_dly2 = "1100001100" else 
X_RD_DATA( 1943 downto 1936 ) when CO1024_dly2 = "1100001101" else 
X_RD_DATA( 1935 downto 1928 ) when CO1024_dly2 = "1100001110" else 
X_RD_DATA( 1927 downto 1920 ) when CO1024_dly2 = "1100001111" else 
X_RD_DATA( 1919 downto 1912 ) when CO1024_dly2 = "1100010000" else 
X_RD_DATA( 1911 downto 1904 ) when CO1024_dly2 = "1100010001" else 
X_RD_DATA( 1903 downto 1896 ) when CO1024_dly2 = "1100010010" else 
X_RD_DATA( 1895 downto 1888 ) when CO1024_dly2 = "1100010011" else 
X_RD_DATA( 1887 downto 1880 ) when CO1024_dly2 = "1100010100" else 
X_RD_DATA( 1879 downto 1872 ) when CO1024_dly2 = "1100010101" else 
X_RD_DATA( 1871 downto 1864 ) when CO1024_dly2 = "1100010110" else 
X_RD_DATA( 1863 downto 1856 ) when CO1024_dly2 = "1100010111" else 
X_RD_DATA( 1855 downto 1848 ) when CO1024_dly2 = "1100011000" else 
X_RD_DATA( 1847 downto 1840 ) when CO1024_dly2 = "1100011001" else 
X_RD_DATA( 1839 downto 1832 ) when CO1024_dly2 = "1100011010" else 
X_RD_DATA( 1831 downto 1824 ) when CO1024_dly2 = "1100011011" else 
X_RD_DATA( 1823 downto 1816 ) when CO1024_dly2 = "1100011100" else 
X_RD_DATA( 1815 downto 1808 ) when CO1024_dly2 = "1100011101" else 
X_RD_DATA( 1807 downto 1800 ) when CO1024_dly2 = "1100011110" else 
X_RD_DATA( 1799 downto 1792 ) when CO1024_dly2 = "1100011111" else 
X_RD_DATA( 1791 downto 1784 ) when CO1024_dly2 = "1100100000" else 
X_RD_DATA( 1783 downto 1776 ) when CO1024_dly2 = "1100100001" else 
X_RD_DATA( 1775 downto 1768 ) when CO1024_dly2 = "1100100010" else 
X_RD_DATA( 1767 downto 1760 ) when CO1024_dly2 = "1100100011" else 
X_RD_DATA( 1759 downto 1752 ) when CO1024_dly2 = "1100100100" else 
X_RD_DATA( 1751 downto 1744 ) when CO1024_dly2 = "1100100101" else 
X_RD_DATA( 1743 downto 1736 ) when CO1024_dly2 = "1100100110" else 
X_RD_DATA( 1735 downto 1728 ) when CO1024_dly2 = "1100100111" else 
X_RD_DATA( 1727 downto 1720 ) when CO1024_dly2 = "1100101000" else 
X_RD_DATA( 1719 downto 1712 ) when CO1024_dly2 = "1100101001" else 
X_RD_DATA( 1711 downto 1704 ) when CO1024_dly2 = "1100101010" else 
X_RD_DATA( 1703 downto 1696 ) when CO1024_dly2 = "1100101011" else 
X_RD_DATA( 1695 downto 1688 ) when CO1024_dly2 = "1100101100" else 
X_RD_DATA( 1687 downto 1680 ) when CO1024_dly2 = "1100101101" else 
X_RD_DATA( 1679 downto 1672 ) when CO1024_dly2 = "1100101110" else 
X_RD_DATA( 1671 downto 1664 ) when CO1024_dly2 = "1100101111" else 
X_RD_DATA( 1663 downto 1656 ) when CO1024_dly2 = "1100110000" else 
X_RD_DATA( 1655 downto 1648 ) when CO1024_dly2 = "1100110001" else 
X_RD_DATA( 1647 downto 1640 ) when CO1024_dly2 = "1100110010" else 
X_RD_DATA( 1639 downto 1632 ) when CO1024_dly2 = "1100110011" else 
X_RD_DATA( 1631 downto 1624 ) when CO1024_dly2 = "1100110100" else 
X_RD_DATA( 1623 downto 1616 ) when CO1024_dly2 = "1100110101" else 
X_RD_DATA( 1615 downto 1608 ) when CO1024_dly2 = "1100110110" else 
X_RD_DATA( 1607 downto 1600 ) when CO1024_dly2 = "1100110111" else 
X_RD_DATA( 1599 downto 1592 ) when CO1024_dly2 = "1100111000" else 
X_RD_DATA( 1591 downto 1584 ) when CO1024_dly2 = "1100111001" else 
X_RD_DATA( 1583 downto 1576 ) when CO1024_dly2 = "1100111010" else 
X_RD_DATA( 1575 downto 1568 ) when CO1024_dly2 = "1100111011" else 
X_RD_DATA( 1567 downto 1560 ) when CO1024_dly2 = "1100111100" else 
X_RD_DATA( 1559 downto 1552 ) when CO1024_dly2 = "1100111101" else 
X_RD_DATA( 1551 downto 1544 ) when CO1024_dly2 = "1100111110" else 
X_RD_DATA( 1543 downto 1536 ) when CO1024_dly2 = "1100111111" else 
X_RD_DATA( 1535 downto 1528 ) when CO1024_dly2 = "1101000000" else 
X_RD_DATA( 1527 downto 1520 ) when CO1024_dly2 = "1101000001" else 
X_RD_DATA( 1519 downto 1512 ) when CO1024_dly2 = "1101000010" else 
X_RD_DATA( 1511 downto 1504 ) when CO1024_dly2 = "1101000011" else 
X_RD_DATA( 1503 downto 1496 ) when CO1024_dly2 = "1101000100" else 
X_RD_DATA( 1495 downto 1488 ) when CO1024_dly2 = "1101000101" else 
X_RD_DATA( 1487 downto 1480 ) when CO1024_dly2 = "1101000110" else 
X_RD_DATA( 1479 downto 1472 ) when CO1024_dly2 = "1101000111" else 
X_RD_DATA( 1471 downto 1464 ) when CO1024_dly2 = "1101001000" else 
X_RD_DATA( 1463 downto 1456 ) when CO1024_dly2 = "1101001001" else 
X_RD_DATA( 1455 downto 1448 ) when CO1024_dly2 = "1101001010" else 
X_RD_DATA( 1447 downto 1440 ) when CO1024_dly2 = "1101001011" else 
X_RD_DATA( 1439 downto 1432 ) when CO1024_dly2 = "1101001100" else 
X_RD_DATA( 1431 downto 1424 ) when CO1024_dly2 = "1101001101" else 
X_RD_DATA( 1423 downto 1416 ) when CO1024_dly2 = "1101001110" else 
X_RD_DATA( 1415 downto 1408 ) when CO1024_dly2 = "1101001111" else 
X_RD_DATA( 1407 downto 1400 ) when CO1024_dly2 = "1101010000" else 
X_RD_DATA( 1399 downto 1392 ) when CO1024_dly2 = "1101010001" else 
X_RD_DATA( 1391 downto 1384 ) when CO1024_dly2 = "1101010010" else 
X_RD_DATA( 1383 downto 1376 ) when CO1024_dly2 = "1101010011" else 
X_RD_DATA( 1375 downto 1368 ) when CO1024_dly2 = "1101010100" else 
X_RD_DATA( 1367 downto 1360 ) when CO1024_dly2 = "1101010101" else 
X_RD_DATA( 1359 downto 1352 ) when CO1024_dly2 = "1101010110" else 
X_RD_DATA( 1351 downto 1344 ) when CO1024_dly2 = "1101010111" else 
X_RD_DATA( 1343 downto 1336 ) when CO1024_dly2 = "1101011000" else 
X_RD_DATA( 1335 downto 1328 ) when CO1024_dly2 = "1101011001" else 
X_RD_DATA( 1327 downto 1320 ) when CO1024_dly2 = "1101011010" else 
X_RD_DATA( 1319 downto 1312 ) when CO1024_dly2 = "1101011011" else 
X_RD_DATA( 1311 downto 1304 ) when CO1024_dly2 = "1101011100" else 
X_RD_DATA( 1303 downto 1296 ) when CO1024_dly2 = "1101011101" else 
X_RD_DATA( 1295 downto 1288 ) when CO1024_dly2 = "1101011110" else 
X_RD_DATA( 1287 downto 1280 ) when CO1024_dly2 = "1101011111" else 
X_RD_DATA( 1279 downto 1272 ) when CO1024_dly2 = "1101100000" else 
X_RD_DATA( 1271 downto 1264 ) when CO1024_dly2 = "1101100001" else 
X_RD_DATA( 1263 downto 1256 ) when CO1024_dly2 = "1101100010" else 
X_RD_DATA( 1255 downto 1248 ) when CO1024_dly2 = "1101100011" else 
X_RD_DATA( 1247 downto 1240 ) when CO1024_dly2 = "1101100100" else 
X_RD_DATA( 1239 downto 1232 ) when CO1024_dly2 = "1101100101" else 
X_RD_DATA( 1231 downto 1224 ) when CO1024_dly2 = "1101100110" else 
X_RD_DATA( 1223 downto 1216 ) when CO1024_dly2 = "1101100111" else 
X_RD_DATA( 1215 downto 1208 ) when CO1024_dly2 = "1101101000" else 
X_RD_DATA( 1207 downto 1200 ) when CO1024_dly2 = "1101101001" else 
X_RD_DATA( 1199 downto 1192 ) when CO1024_dly2 = "1101101010" else 
X_RD_DATA( 1191 downto 1184 ) when CO1024_dly2 = "1101101011" else 
X_RD_DATA( 1183 downto 1176 ) when CO1024_dly2 = "1101101100" else 
X_RD_DATA( 1175 downto 1168 ) when CO1024_dly2 = "1101101101" else 
X_RD_DATA( 1167 downto 1160 ) when CO1024_dly2 = "1101101110" else 
X_RD_DATA( 1159 downto 1152 ) when CO1024_dly2 = "1101101111" else 
X_RD_DATA( 1151 downto 1144 ) when CO1024_dly2 = "1101110000" else 
X_RD_DATA( 1143 downto 1136 ) when CO1024_dly2 = "1101110001" else 
X_RD_DATA( 1135 downto 1128 ) when CO1024_dly2 = "1101110010" else 
X_RD_DATA( 1127 downto 1120 ) when CO1024_dly2 = "1101110011" else 
X_RD_DATA( 1119 downto 1112 ) when CO1024_dly2 = "1101110100" else 
X_RD_DATA( 1111 downto 1104 ) when CO1024_dly2 = "1101110101" else 
X_RD_DATA( 1103 downto 1096 ) when CO1024_dly2 = "1101110110" else 
X_RD_DATA( 1095 downto 1088 ) when CO1024_dly2 = "1101110111" else 
X_RD_DATA( 1087 downto 1080 ) when CO1024_dly2 = "1101111000" else 
X_RD_DATA( 1079 downto 1072 ) when CO1024_dly2 = "1101111001" else 
X_RD_DATA( 1071 downto 1064 ) when CO1024_dly2 = "1101111010" else 
X_RD_DATA( 1063 downto 1056 ) when CO1024_dly2 = "1101111011" else 
X_RD_DATA( 1055 downto 1048 ) when CO1024_dly2 = "1101111100" else 
X_RD_DATA( 1047 downto 1040 ) when CO1024_dly2 = "1101111101" else 
X_RD_DATA( 1039 downto 1032 ) when CO1024_dly2 = "1101111110" else 
X_RD_DATA( 1031 downto 1024 ) when CO1024_dly2 = "1101111111" else 
X_RD_DATA( 1023 downto 1016 ) when CO1024_dly2 = "1110000000" else 
X_RD_DATA( 1015 downto 1008 ) when CO1024_dly2 = "1110000001" else 
X_RD_DATA( 1007 downto 1000 ) when CO1024_dly2 = "1110000010" else 
X_RD_DATA( 999 downto 992 ) when CO1024_dly2 = "1110000011" else 
X_RD_DATA( 991 downto 984 ) when CO1024_dly2 = "1110000100" else 
X_RD_DATA( 983 downto 976 ) when CO1024_dly2 = "1110000101" else 
X_RD_DATA( 975 downto 968 ) when CO1024_dly2 = "1110000110" else 
X_RD_DATA( 967 downto 960 ) when CO1024_dly2 = "1110000111" else 
X_RD_DATA( 959 downto 952 ) when CO1024_dly2 = "1110001000" else 
X_RD_DATA( 951 downto 944 ) when CO1024_dly2 = "1110001001" else 
X_RD_DATA( 943 downto 936 ) when CO1024_dly2 = "1110001010" else 
X_RD_DATA( 935 downto 928 ) when CO1024_dly2 = "1110001011" else 
X_RD_DATA( 927 downto 920 ) when CO1024_dly2 = "1110001100" else 
X_RD_DATA( 919 downto 912 ) when CO1024_dly2 = "1110001101" else 
X_RD_DATA( 911 downto 904 ) when CO1024_dly2 = "1110001110" else 
X_RD_DATA( 903 downto 896 ) when CO1024_dly2 = "1110001111" else 
X_RD_DATA( 895 downto 888 ) when CO1024_dly2 = "1110010000" else 
X_RD_DATA( 887 downto 880 ) when CO1024_dly2 = "1110010001" else 
X_RD_DATA( 879 downto 872 ) when CO1024_dly2 = "1110010010" else 
X_RD_DATA( 871 downto 864 ) when CO1024_dly2 = "1110010011" else 
X_RD_DATA( 863 downto 856 ) when CO1024_dly2 = "1110010100" else 
X_RD_DATA( 855 downto 848 ) when CO1024_dly2 = "1110010101" else 
X_RD_DATA( 847 downto 840 ) when CO1024_dly2 = "1110010110" else 
X_RD_DATA( 839 downto 832 ) when CO1024_dly2 = "1110010111" else 
X_RD_DATA( 831 downto 824 ) when CO1024_dly2 = "1110011000" else 
X_RD_DATA( 823 downto 816 ) when CO1024_dly2 = "1110011001" else 
X_RD_DATA( 815 downto 808 ) when CO1024_dly2 = "1110011010" else 
X_RD_DATA( 807 downto 800 ) when CO1024_dly2 = "1110011011" else 
X_RD_DATA( 799 downto 792 ) when CO1024_dly2 = "1110011100" else 
X_RD_DATA( 791 downto 784 ) when CO1024_dly2 = "1110011101" else 
X_RD_DATA( 783 downto 776 ) when CO1024_dly2 = "1110011110" else 
X_RD_DATA( 775 downto 768 ) when CO1024_dly2 = "1110011111" else 
X_RD_DATA( 767 downto 760 ) when CO1024_dly2 = "1110100000" else 
X_RD_DATA( 759 downto 752 ) when CO1024_dly2 = "1110100001" else 
X_RD_DATA( 751 downto 744 ) when CO1024_dly2 = "1110100010" else 
X_RD_DATA( 743 downto 736 ) when CO1024_dly2 = "1110100011" else 
X_RD_DATA( 735 downto 728 ) when CO1024_dly2 = "1110100100" else 
X_RD_DATA( 727 downto 720 ) when CO1024_dly2 = "1110100101" else 
X_RD_DATA( 719 downto 712 ) when CO1024_dly2 = "1110100110" else 
X_RD_DATA( 711 downto 704 ) when CO1024_dly2 = "1110100111" else 
X_RD_DATA( 703 downto 696 ) when CO1024_dly2 = "1110101000" else 
X_RD_DATA( 695 downto 688 ) when CO1024_dly2 = "1110101001" else 
X_RD_DATA( 687 downto 680 ) when CO1024_dly2 = "1110101010" else 
X_RD_DATA( 679 downto 672 ) when CO1024_dly2 = "1110101011" else 
X_RD_DATA( 671 downto 664 ) when CO1024_dly2 = "1110101100" else 
X_RD_DATA( 663 downto 656 ) when CO1024_dly2 = "1110101101" else 
X_RD_DATA( 655 downto 648 ) when CO1024_dly2 = "1110101110" else 
X_RD_DATA( 647 downto 640 ) when CO1024_dly2 = "1110101111" else 
X_RD_DATA( 639 downto 632 ) when CO1024_dly2 = "1110110000" else 
X_RD_DATA( 631 downto 624 ) when CO1024_dly2 = "1110110001" else 
X_RD_DATA( 623 downto 616 ) when CO1024_dly2 = "1110110010" else 
X_RD_DATA( 615 downto 608 ) when CO1024_dly2 = "1110110011" else 
X_RD_DATA( 607 downto 600 ) when CO1024_dly2 = "1110110100" else 
X_RD_DATA( 599 downto 592 ) when CO1024_dly2 = "1110110101" else 
X_RD_DATA( 591 downto 584 ) when CO1024_dly2 = "1110110110" else 
X_RD_DATA( 583 downto 576 ) when CO1024_dly2 = "1110110111" else 
X_RD_DATA( 575 downto 568 ) when CO1024_dly2 = "1110111000" else 
X_RD_DATA( 567 downto 560 ) when CO1024_dly2 = "1110111001" else 
X_RD_DATA( 559 downto 552 ) when CO1024_dly2 = "1110111010" else 
X_RD_DATA( 551 downto 544 ) when CO1024_dly2 = "1110111011" else 
X_RD_DATA( 543 downto 536 ) when CO1024_dly2 = "1110111100" else 
X_RD_DATA( 535 downto 528 ) when CO1024_dly2 = "1110111101" else 
X_RD_DATA( 527 downto 520 ) when CO1024_dly2 = "1110111110" else 
X_RD_DATA( 519 downto 512 ) when CO1024_dly2 = "1110111111" else 
X_RD_DATA( 511 downto 504 ) when CO1024_dly2 = "1111000000" else 
X_RD_DATA( 503 downto 496 ) when CO1024_dly2 = "1111000001" else 
X_RD_DATA( 495 downto 488 ) when CO1024_dly2 = "1111000010" else 
X_RD_DATA( 487 downto 480 ) when CO1024_dly2 = "1111000011" else 
X_RD_DATA( 479 downto 472 ) when CO1024_dly2 = "1111000100" else 
X_RD_DATA( 471 downto 464 ) when CO1024_dly2 = "1111000101" else 
X_RD_DATA( 463 downto 456 ) when CO1024_dly2 = "1111000110" else 
X_RD_DATA( 455 downto 448 ) when CO1024_dly2 = "1111000111" else 
X_RD_DATA( 447 downto 440 ) when CO1024_dly2 = "1111001000" else 
X_RD_DATA( 439 downto 432 ) when CO1024_dly2 = "1111001001" else 
X_RD_DATA( 431 downto 424 ) when CO1024_dly2 = "1111001010" else 
X_RD_DATA( 423 downto 416 ) when CO1024_dly2 = "1111001011" else 
X_RD_DATA( 415 downto 408 ) when CO1024_dly2 = "1111001100" else 
X_RD_DATA( 407 downto 400 ) when CO1024_dly2 = "1111001101" else 
X_RD_DATA( 399 downto 392 ) when CO1024_dly2 = "1111001110" else 
X_RD_DATA( 391 downto 384 ) when CO1024_dly2 = "1111001111" else 
X_RD_DATA( 383 downto 376 ) when CO1024_dly2 = "1111010000" else 
X_RD_DATA( 375 downto 368 ) when CO1024_dly2 = "1111010001" else 
X_RD_DATA( 367 downto 360 ) when CO1024_dly2 = "1111010010" else 
X_RD_DATA( 359 downto 352 ) when CO1024_dly2 = "1111010011" else 
X_RD_DATA( 351 downto 344 ) when CO1024_dly2 = "1111010100" else 
X_RD_DATA( 343 downto 336 ) when CO1024_dly2 = "1111010101" else 
X_RD_DATA( 335 downto 328 ) when CO1024_dly2 = "1111010110" else 
X_RD_DATA( 327 downto 320 ) when CO1024_dly2 = "1111010111" else 
X_RD_DATA( 319 downto 312 ) when CO1024_dly2 = "1111011000" else 
X_RD_DATA( 311 downto 304 ) when CO1024_dly2 = "1111011001" else 
X_RD_DATA( 303 downto 296 ) when CO1024_dly2 = "1111011010" else 
X_RD_DATA( 295 downto 288 ) when CO1024_dly2 = "1111011011" else 
X_RD_DATA( 287 downto 280 ) when CO1024_dly2 = "1111011100" else 
X_RD_DATA( 279 downto 272 ) when CO1024_dly2 = "1111011101" else 
X_RD_DATA( 271 downto 264 ) when CO1024_dly2 = "1111011110" else 
X_RD_DATA( 263 downto 256 ) when CO1024_dly2 = "1111011111" else 
X_RD_DATA( 255 downto 248 ) when CO1024_dly2 = "1111100000" else 
X_RD_DATA( 247 downto 240 ) when CO1024_dly2 = "1111100001" else 
X_RD_DATA( 239 downto 232 ) when CO1024_dly2 = "1111100010" else 
X_RD_DATA( 231 downto 224 ) when CO1024_dly2 = "1111100011" else 
X_RD_DATA( 223 downto 216 ) when CO1024_dly2 = "1111100100" else 
X_RD_DATA( 215 downto 208 ) when CO1024_dly2 = "1111100101" else 
X_RD_DATA( 207 downto 200 ) when CO1024_dly2 = "1111100110" else 
X_RD_DATA( 199 downto 192 ) when CO1024_dly2 = "1111100111" else 
X_RD_DATA( 191 downto 184 ) when CO1024_dly2 = "1111101000" else 
X_RD_DATA( 183 downto 176 ) when CO1024_dly2 = "1111101001" else 
X_RD_DATA( 175 downto 168 ) when CO1024_dly2 = "1111101010" else 
X_RD_DATA( 167 downto 160 ) when CO1024_dly2 = "1111101011" else 
X_RD_DATA( 159 downto 152 ) when CO1024_dly2 = "1111101100" else 
X_RD_DATA( 151 downto 144 ) when CO1024_dly2 = "1111101101" else 
X_RD_DATA( 143 downto 136 ) when CO1024_dly2 = "1111101110" else 
X_RD_DATA( 135 downto 128 ) when CO1024_dly2 = "1111101111" else 
X_RD_DATA( 127 downto 120 ) when CO1024_dly2 = "1111110000" else 
X_RD_DATA( 119 downto 112 ) when CO1024_dly2 = "1111110001" else 
X_RD_DATA( 111 downto 104 ) when CO1024_dly2 = "1111110010" else 
X_RD_DATA( 103 downto 96 ) when CO1024_dly2 = "1111110011" else 
X_RD_DATA( 95 downto 88 ) when CO1024_dly2 = "1111110100" else 
X_RD_DATA( 87 downto 80 ) when CO1024_dly2 = "1111110101" else 
X_RD_DATA( 79 downto 72 ) when CO1024_dly2 = "1111110110" else 
X_RD_DATA( 71 downto 64 ) when CO1024_dly2 = "1111110111" else 
X_RD_DATA( 63 downto 56 ) when CO1024_dly2 = "1111111000" else 
X_RD_DATA( 55 downto 48 ) when CO1024_dly2 = "1111111001" else 
X_RD_DATA( 47 downto 40 ) when CO1024_dly2 = "1111111010" else 
X_RD_DATA( 39 downto 32 ) when CO1024_dly2 = "1111111011" else 
X_RD_DATA( 31 downto 24 ) when CO1024_dly2 = "1111111100" else 
X_RD_DATA( 23 downto 16 ) when CO1024_dly2 = "1111111101" else 
X_RD_DATA( 15 downto 8 ) when CO1024_dly2 = "1111111110" else 
X_RD_DATA( 7 downto 0 ); --- when CO1024_dly2 = "1111111111" else 


CW2Addr_L <= CO9;
CW2Addr_H <= ('0' & CO1024(9 downto 0) & "000") + ("00" & CO1024); -- CO1024 x 9
CW2Addr_prev <= CW2Addr_H + ("0000000000" & CW2Addr_L);

--CW2Addr_prev <= (CO1024); -- CO1024 x 1

---------------------------------------------------------
process( RST_N, CLK )
begin
        if( RST_N = '0' ) then
                CW2Addr <= (others => '0');
        elsif( CLK'event and CLK = '1' ) then
                CW2Addr <= CW2Addr_prev;
        end if; 
end process;


---------------------------------------------------------
enb_000 <= '1' when CW2Sel = 1 and CO1024 = 1023 else '0';
enb_001 <= '1' when CW2Sel = 2 and CO1024 = 1023 else '0';
enb_002 <= '1' when CW2Sel = 3 and CO1024 = 1023 else '0';
enb_003 <= '1' when CW2Sel = 4 and CO1024 = 1023 else '0';
enb_004 <= '1' when CW2Sel = 5 and CO1024 = 1023 else '0';
enb_005 <= '1' when CW2Sel = 6 and CO1024 = 1023 else '0';
enb_006 <= '1' when CW2Sel = 7 and CO1024 = 1023 else '0';
enb_007 <= '1' when CW2Sel = 0 and CO1024 = 1023 else '0';

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_000_000 <= Relu_000;
    elsif(enb_001 = '1') then reg_000_001 <= Relu_000;
    elsif(enb_002 = '1') then reg_000_002 <= Relu_000;
    elsif(enb_003 = '1') then reg_000_003 <= Relu_000;
    elsif(enb_004 = '1') then reg_000_004 <= Relu_000;
    elsif(enb_005 = '1') then reg_000_005 <= Relu_000;
    elsif(enb_006 = '1') then reg_000_006 <= Relu_000;
    elsif(enb_007 = '1') then reg_000_007 <= Relu_000;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_001_000 <= Relu_001;
    elsif(enb_001 = '1') then reg_001_001 <= Relu_001;
    elsif(enb_002 = '1') then reg_001_002 <= Relu_001;
    elsif(enb_003 = '1') then reg_001_003 <= Relu_001;
    elsif(enb_004 = '1') then reg_001_004 <= Relu_001;
    elsif(enb_005 = '1') then reg_001_005 <= Relu_001;
    elsif(enb_006 = '1') then reg_001_006 <= Relu_001;
    elsif(enb_007 = '1') then reg_001_007 <= Relu_001;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_002_000 <= Relu_002;
    elsif(enb_001 = '1') then reg_002_001 <= Relu_002;
    elsif(enb_002 = '1') then reg_002_002 <= Relu_002;
    elsif(enb_003 = '1') then reg_002_003 <= Relu_002;
    elsif(enb_004 = '1') then reg_002_004 <= Relu_002;
    elsif(enb_005 = '1') then reg_002_005 <= Relu_002;
    elsif(enb_006 = '1') then reg_002_006 <= Relu_002;
    elsif(enb_007 = '1') then reg_002_007 <= Relu_002;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_003_000 <= Relu_003;
    elsif(enb_001 = '1') then reg_003_001 <= Relu_003;
    elsif(enb_002 = '1') then reg_003_002 <= Relu_003;
    elsif(enb_003 = '1') then reg_003_003 <= Relu_003;
    elsif(enb_004 = '1') then reg_003_004 <= Relu_003;
    elsif(enb_005 = '1') then reg_003_005 <= Relu_003;
    elsif(enb_006 = '1') then reg_003_006 <= Relu_003;
    elsif(enb_007 = '1') then reg_003_007 <= Relu_003;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_004_000 <= Relu_004;
    elsif(enb_001 = '1') then reg_004_001 <= Relu_004;
    elsif(enb_002 = '1') then reg_004_002 <= Relu_004;
    elsif(enb_003 = '1') then reg_004_003 <= Relu_004;
    elsif(enb_004 = '1') then reg_004_004 <= Relu_004;
    elsif(enb_005 = '1') then reg_004_005 <= Relu_004;
    elsif(enb_006 = '1') then reg_004_006 <= Relu_004;
    elsif(enb_007 = '1') then reg_004_007 <= Relu_004;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_005_000 <= Relu_005;
    elsif(enb_001 = '1') then reg_005_001 <= Relu_005;
    elsif(enb_002 = '1') then reg_005_002 <= Relu_005;
    elsif(enb_003 = '1') then reg_005_003 <= Relu_005;
    elsif(enb_004 = '1') then reg_005_004 <= Relu_005;
    elsif(enb_005 = '1') then reg_005_005 <= Relu_005;
    elsif(enb_006 = '1') then reg_005_006 <= Relu_005;
    elsif(enb_007 = '1') then reg_005_007 <= Relu_005;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_006_000 <= Relu_006;
    elsif(enb_001 = '1') then reg_006_001 <= Relu_006;
    elsif(enb_002 = '1') then reg_006_002 <= Relu_006;
    elsif(enb_003 = '1') then reg_006_003 <= Relu_006;
    elsif(enb_004 = '1') then reg_006_004 <= Relu_006;
    elsif(enb_005 = '1') then reg_006_005 <= Relu_006;
    elsif(enb_006 = '1') then reg_006_006 <= Relu_006;
    elsif(enb_007 = '1') then reg_006_007 <= Relu_006;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_007_000 <= Relu_007;
    elsif(enb_001 = '1') then reg_007_001 <= Relu_007;
    elsif(enb_002 = '1') then reg_007_002 <= Relu_007;
    elsif(enb_003 = '1') then reg_007_003 <= Relu_007;
    elsif(enb_004 = '1') then reg_007_004 <= Relu_007;
    elsif(enb_005 = '1') then reg_007_005 <= Relu_007;
    elsif(enb_006 = '1') then reg_007_006 <= Relu_007;
    elsif(enb_007 = '1') then reg_007_007 <= Relu_007;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_008_000 <= Relu_008;
    elsif(enb_001 = '1') then reg_008_001 <= Relu_008;
    elsif(enb_002 = '1') then reg_008_002 <= Relu_008;
    elsif(enb_003 = '1') then reg_008_003 <= Relu_008;
    elsif(enb_004 = '1') then reg_008_004 <= Relu_008;
    elsif(enb_005 = '1') then reg_008_005 <= Relu_008;
    elsif(enb_006 = '1') then reg_008_006 <= Relu_008;
    elsif(enb_007 = '1') then reg_008_007 <= Relu_008;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_009_000 <= Relu_009;
    elsif(enb_001 = '1') then reg_009_001 <= Relu_009;
    elsif(enb_002 = '1') then reg_009_002 <= Relu_009;
    elsif(enb_003 = '1') then reg_009_003 <= Relu_009;
    elsif(enb_004 = '1') then reg_009_004 <= Relu_009;
    elsif(enb_005 = '1') then reg_009_005 <= Relu_009;
    elsif(enb_006 = '1') then reg_009_006 <= Relu_009;
    elsif(enb_007 = '1') then reg_009_007 <= Relu_009;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_010_000 <= Relu_010;
    elsif(enb_001 = '1') then reg_010_001 <= Relu_010;
    elsif(enb_002 = '1') then reg_010_002 <= Relu_010;
    elsif(enb_003 = '1') then reg_010_003 <= Relu_010;
    elsif(enb_004 = '1') then reg_010_004 <= Relu_010;
    elsif(enb_005 = '1') then reg_010_005 <= Relu_010;
    elsif(enb_006 = '1') then reg_010_006 <= Relu_010;
    elsif(enb_007 = '1') then reg_010_007 <= Relu_010;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_011_000 <= Relu_011;
    elsif(enb_001 = '1') then reg_011_001 <= Relu_011;
    elsif(enb_002 = '1') then reg_011_002 <= Relu_011;
    elsif(enb_003 = '1') then reg_011_003 <= Relu_011;
    elsif(enb_004 = '1') then reg_011_004 <= Relu_011;
    elsif(enb_005 = '1') then reg_011_005 <= Relu_011;
    elsif(enb_006 = '1') then reg_011_006 <= Relu_011;
    elsif(enb_007 = '1') then reg_011_007 <= Relu_011;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_012_000 <= Relu_012;
    elsif(enb_001 = '1') then reg_012_001 <= Relu_012;
    elsif(enb_002 = '1') then reg_012_002 <= Relu_012;
    elsif(enb_003 = '1') then reg_012_003 <= Relu_012;
    elsif(enb_004 = '1') then reg_012_004 <= Relu_012;
    elsif(enb_005 = '1') then reg_012_005 <= Relu_012;
    elsif(enb_006 = '1') then reg_012_006 <= Relu_012;
    elsif(enb_007 = '1') then reg_012_007 <= Relu_012;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_013_000 <= Relu_013;
    elsif(enb_001 = '1') then reg_013_001 <= Relu_013;
    elsif(enb_002 = '1') then reg_013_002 <= Relu_013;
    elsif(enb_003 = '1') then reg_013_003 <= Relu_013;
    elsif(enb_004 = '1') then reg_013_004 <= Relu_013;
    elsif(enb_005 = '1') then reg_013_005 <= Relu_013;
    elsif(enb_006 = '1') then reg_013_006 <= Relu_013;
    elsif(enb_007 = '1') then reg_013_007 <= Relu_013;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_014_000 <= Relu_014;
    elsif(enb_001 = '1') then reg_014_001 <= Relu_014;
    elsif(enb_002 = '1') then reg_014_002 <= Relu_014;
    elsif(enb_003 = '1') then reg_014_003 <= Relu_014;
    elsif(enb_004 = '1') then reg_014_004 <= Relu_014;
    elsif(enb_005 = '1') then reg_014_005 <= Relu_014;
    elsif(enb_006 = '1') then reg_014_006 <= Relu_014;
    elsif(enb_007 = '1') then reg_014_007 <= Relu_014;
    end if;
  end if;  
end process;

process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_015_000 <= Relu_015;
    elsif(enb_001 = '1') then reg_015_001 <= Relu_015;
    elsif(enb_002 = '1') then reg_015_002 <= Relu_015;
    elsif(enb_003 = '1') then reg_015_003 <= Relu_015;
    elsif(enb_004 = '1') then reg_015_004 <= Relu_015;
    elsif(enb_005 = '1') then reg_015_005 <= Relu_015;
    elsif(enb_006 = '1') then reg_015_006 <= Relu_015;
    elsif(enb_007 = '1') then reg_015_007 <= Relu_015;
    end if;
  end if;  
end process;


-------------------------------------------------------
ROM_CW2_inst : ROM_CW2 port map (
  CLK => CLK,
  ADDR => CW2Addr,
  DATA => CW2Data );

---------------------------------------------------------
coeff_000 <= 
CW2Data( 377 downto 375 ) when CW2Sel = "00000000" else 
CW2Data( 374 downto 372 ) when CW2Sel = "00000001" else 
CW2Data( 371 downto 369 ) when CW2Sel = "00000010" else 
CW2Data( 368 downto 366 ) when CW2Sel = "00000011" else 
CW2Data( 365 downto 363 ) when CW2Sel = "00000100" else 
CW2Data( 362 downto 360 ) when CW2Sel = "00000101" else 
CW2Data( 359 downto 357 ) when CW2Sel = "00000110" else 
CW2Data( 356 downto 354 ) when CW2Sel = "00000111" ; 

coeff_001 <= 
CW2Data( 353 downto 351 ) when CW2Sel = "00000000" else 
CW2Data( 350 downto 348 ) when CW2Sel = "00000001" else 
CW2Data( 347 downto 345 ) when CW2Sel = "00000010" else 
CW2Data( 344 downto 342 ) when CW2Sel = "00000011" else 
CW2Data( 341 downto 339 ) when CW2Sel = "00000100" else 
CW2Data( 338 downto 336 ) when CW2Sel = "00000101" else 
CW2Data( 335 downto 333 ) when CW2Sel = "00000110" else 
CW2Data( 332 downto 330 ) when CW2Sel = "00000111" ; 

coeff_002 <= 
CW2Data( 329 downto 327 ) when CW2Sel = "00000000" else 
CW2Data( 326 downto 324 ) when CW2Sel = "00000001" else 
CW2Data( 323 downto 321 ) when CW2Sel = "00000010" else 
CW2Data( 320 downto 318 ) when CW2Sel = "00000011" else 
CW2Data( 317 downto 315 ) when CW2Sel = "00000100" else 
CW2Data( 314 downto 312 ) when CW2Sel = "00000101" else 
CW2Data( 311 downto 309 ) when CW2Sel = "00000110" else 
CW2Data( 308 downto 306 ) when CW2Sel = "00000111" ; 

coeff_003 <= 
CW2Data( 305 downto 303 ) when CW2Sel = "00000000" else 
CW2Data( 302 downto 300 ) when CW2Sel = "00000001" else 
CW2Data( 299 downto 297 ) when CW2Sel = "00000010" else 
CW2Data( 296 downto 294 ) when CW2Sel = "00000011" else 
CW2Data( 293 downto 291 ) when CW2Sel = "00000100" else 
CW2Data( 290 downto 288 ) when CW2Sel = "00000101" else 
CW2Data( 287 downto 285 ) when CW2Sel = "00000110" else 
CW2Data( 284 downto 282 ) when CW2Sel = "00000111" ; 

coeff_004 <= 
CW2Data( 281 downto 279 ) when CW2Sel = "00000000" else 
CW2Data( 278 downto 276 ) when CW2Sel = "00000001" else 
CW2Data( 275 downto 273 ) when CW2Sel = "00000010" else 
CW2Data( 272 downto 270 ) when CW2Sel = "00000011" else 
CW2Data( 269 downto 267 ) when CW2Sel = "00000100" else 
CW2Data( 266 downto 264 ) when CW2Sel = "00000101" else 
CW2Data( 263 downto 261 ) when CW2Sel = "00000110" else 
CW2Data( 260 downto 258 ) when CW2Sel = "00000111" ; 

coeff_005 <= 
CW2Data( 257 downto 255 ) when CW2Sel = "00000000" else 
CW2Data( 254 downto 252 ) when CW2Sel = "00000001" else 
CW2Data( 251 downto 249 ) when CW2Sel = "00000010" else 
CW2Data( 248 downto 246 ) when CW2Sel = "00000011" else 
CW2Data( 245 downto 243 ) when CW2Sel = "00000100" else 
CW2Data( 242 downto 240 ) when CW2Sel = "00000101" else 
CW2Data( 239 downto 237 ) when CW2Sel = "00000110" else 
CW2Data( 236 downto 234 ) when CW2Sel = "00000111" ; 

coeff_006 <= 
CW2Data( 233 downto 231 ) when CW2Sel = "00000000" else 
CW2Data( 230 downto 228 ) when CW2Sel = "00000001" else 
CW2Data( 227 downto 225 ) when CW2Sel = "00000010" else 
CW2Data( 224 downto 222 ) when CW2Sel = "00000011" else 
CW2Data( 221 downto 219 ) when CW2Sel = "00000100" else 
CW2Data( 218 downto 216 ) when CW2Sel = "00000101" else 
CW2Data( 215 downto 213 ) when CW2Sel = "00000110" else 
CW2Data( 212 downto 210 ) when CW2Sel = "00000111" ; 

coeff_007 <= 
CW2Data( 209 downto 207 ) when CW2Sel = "00000000" else 
CW2Data( 206 downto 204 ) when CW2Sel = "00000001" else 
CW2Data( 203 downto 201 ) when CW2Sel = "00000010" else 
CW2Data( 200 downto 198 ) when CW2Sel = "00000011" else 
CW2Data( 197 downto 195 ) when CW2Sel = "00000100" else 
CW2Data( 194 downto 192 ) when CW2Sel = "00000101" else 
CW2Data( 191 downto 189 ) when CW2Sel = "00000110" else 
CW2Data( 188 downto 186 ) when CW2Sel = "00000111" ; 

coeff_008 <= 
CW2Data( 185 downto 183 ) when CW2Sel = "00000000" else 
CW2Data( 182 downto 180 ) when CW2Sel = "00000001" else 
CW2Data( 179 downto 177 ) when CW2Sel = "00000010" else 
CW2Data( 176 downto 174 ) when CW2Sel = "00000011" else 
CW2Data( 173 downto 171 ) when CW2Sel = "00000100" else 
CW2Data( 170 downto 168 ) when CW2Sel = "00000101" else 
CW2Data( 167 downto 165 ) when CW2Sel = "00000110" else 
CW2Data( 164 downto 162 ) when CW2Sel = "00000111" ; 

coeff_009 <= 
CW2Data( 161 downto 159 ) when CW2Sel = "00000000" else 
CW2Data( 158 downto 156 ) when CW2Sel = "00000001" else 
CW2Data( 155 downto 153 ) when CW2Sel = "00000010" else 
CW2Data( 152 downto 150 ) when CW2Sel = "00000011" else 
CW2Data( 149 downto 147 ) when CW2Sel = "00000100" else 
CW2Data( 146 downto 144 ) when CW2Sel = "00000101" else 
CW2Data( 143 downto 141 ) when CW2Sel = "00000110" else 
CW2Data( 140 downto 138 ) when CW2Sel = "00000111" ; 

coeff_010 <= 
CW2Data( 137 downto 135 ) when CW2Sel = "00000000" else 
CW2Data( 134 downto 132 ) when CW2Sel = "00000001" else 
CW2Data( 131 downto 129 ) when CW2Sel = "00000010" else 
CW2Data( 128 downto 126 ) when CW2Sel = "00000011" else 
CW2Data( 125 downto 123 ) when CW2Sel = "00000100" else 
CW2Data( 122 downto 120 ) when CW2Sel = "00000101" else 
CW2Data( 119 downto 117 ) when CW2Sel = "00000110" else 
CW2Data( 116 downto 114 ) when CW2Sel = "00000111" ; 

coeff_011 <= 
CW2Data( 113 downto 111 ) when CW2Sel = "00000000" else 
CW2Data( 110 downto 108 ) when CW2Sel = "00000001" else 
CW2Data( 107 downto 105 ) when CW2Sel = "00000010" else 
CW2Data( 104 downto 102 ) when CW2Sel = "00000011" else 
CW2Data( 101 downto 99 ) when CW2Sel = "00000100" else 
CW2Data( 98 downto 96 ) when CW2Sel = "00000101" else 
CW2Data( 95 downto 93 ) when CW2Sel = "00000110" else 
CW2Data( 92 downto 90 ) when CW2Sel = "00000111" ; 

coeff_012 <= 
CW2Data( 89 downto 87 ) when CW2Sel = "00000000" else 
CW2Data( 86 downto 84 ) when CW2Sel = "00000001" else 
CW2Data( 83 downto 81 ) when CW2Sel = "00000010" else 
CW2Data( 80 downto 78 ) when CW2Sel = "00000011" else 
CW2Data( 77 downto 75 ) when CW2Sel = "00000100" else 
CW2Data( 74 downto 72 ) when CW2Sel = "00000101" else 
CW2Data( 71 downto 69 ) when CW2Sel = "00000110" else 
CW2Data( 68 downto 66 ) when CW2Sel = "00000111" ; 

coeff_013 <= 
CW2Data( 65 downto 63 ) when CW2Sel = "00000000" else 
CW2Data( 62 downto 60 ) when CW2Sel = "00000001" else 
CW2Data( 59 downto 57 ) when CW2Sel = "00000010" else 
CW2Data( 56 downto 54 ) when CW2Sel = "00000011" else 
CW2Data( 53 downto 51 ) when CW2Sel = "00000100" else 
CW2Data( 50 downto 48 ) when CW2Sel = "00000101" else 
CW2Data( 47 downto 45 ) when CW2Sel = "00000110" else 
CW2Data( 44 downto 42 ) when CW2Sel = "00000111" ; 

coeff_014 <= 
CW2Data( 41 downto 39 ) when CW2Sel = "00000000" else 
CW2Data( 38 downto 36 ) when CW2Sel = "00000001" else 
CW2Data( 35 downto 33 ) when CW2Sel = "00000010" else 
CW2Data( 32 downto 30 ) when CW2Sel = "00000011" else 
CW2Data( 29 downto 27 ) when CW2Sel = "00000100" else 
CW2Data( 26 downto 24 ) when CW2Sel = "00000101" else 
CW2Data( 23 downto 21 ) when CW2Sel = "00000110" else 
CW2Data( 20 downto 18 ) when CW2Sel = "00000111" ; 

coeff_015 <= 
CW2Data( 17 downto 15 ) when CW2Sel = "00000000" else 
CW2Data( 14 downto 12 ) when CW2Sel = "00000001" else 
CW2Data( 11 downto 9 ) when CW2Sel = "00000010" else 
CW2Data( 8 downto 6 ) when CW2Sel = "00000011" else 
CW2Data( 5 downto 3 ) when CW2Sel = "00000100" else 
CW2Data( 2 downto 0 ) ; ---- when CW2Sel = "00000101" ; 

---------------------------------------------------------
dotpro_conf2_000 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_000,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_000 );

dotpro_conf2_001 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_001,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_001 );

dotpro_conf2_002 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_002,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_002 );

dotpro_conf2_003 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_003,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_003 );

dotpro_conf2_004 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_004,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_004 );

dotpro_conf2_005 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_005,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_005 );

dotpro_conf2_006 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_006,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_006 );

dotpro_conf2_007 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_007,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_007 );

dotpro_conf2_008 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_008,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_008 );

dotpro_conf2_009 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_009,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_009 );

dotpro_conf2_010 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_010,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_010 );

dotpro_conf2_011 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_011,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_011 );

dotpro_conf2_012 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_012,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_012 );

dotpro_conf2_013 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_013,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_013 );

dotpro_conf2_014 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_014,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_014 );

dotpro_conf2_015 : dotpro_conf2 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_015,
  CO9is1 => CO9is1,
  CO1024is0 => CO1024is0,
  MATMULOUT => mac_015 );

-------------------------------------------------------
  truncate_000 <= mac_000(16 downto 4);
  truncate_001 <= mac_001(16 downto 4);
  truncate_002 <= mac_002(16 downto 4);
  truncate_003 <= mac_003(16 downto 4);
  truncate_004 <= mac_004(16 downto 4);
  truncate_005 <= mac_005(16 downto 4);
  truncate_006 <= mac_006(16 downto 4);
  truncate_007 <= mac_007(16 downto 4);
  truncate_008 <= mac_008(16 downto 4);
  truncate_009 <= mac_009(16 downto 4);
  truncate_010 <= mac_010(16 downto 4);
  truncate_011 <= mac_011(16 downto 4);
  truncate_012 <= mac_012(16 downto 4);
  truncate_013 <= mac_013(16 downto 4);
  truncate_014 <= mac_014(16 downto 4);
  truncate_015 <= mac_015(16 downto 4);

---------------------------------------------------------
  Relu_000 <= "100000000000" when truncate_000 < -2048 else
             "011111111111" when truncate_000 > 2047 else
             truncate_000(11 downto 0);

  Relu_001 <= "100000000000" when truncate_001 < -2048 else
             "011111111111" when truncate_001 > 2047 else
             truncate_001(11 downto 0);

  Relu_002 <= "100000000000" when truncate_002 < -2048 else
             "011111111111" when truncate_002 > 2047 else
             truncate_002(11 downto 0);

  Relu_003 <= "100000000000" when truncate_003 < -2048 else
             "011111111111" when truncate_003 > 2047 else
             truncate_003(11 downto 0);

  Relu_004 <= "100000000000" when truncate_004 < -2048 else
             "011111111111" when truncate_004 > 2047 else
             truncate_004(11 downto 0);

  Relu_005 <= "100000000000" when truncate_005 < -2048 else
             "011111111111" when truncate_005 > 2047 else
             truncate_005(11 downto 0);

  Relu_006 <= "100000000000" when truncate_006 < -2048 else
             "011111111111" when truncate_006 > 2047 else
             truncate_006(11 downto 0);

  Relu_007 <= "100000000000" when truncate_007 < -2048 else
             "011111111111" when truncate_007 > 2047 else
             truncate_007(11 downto 0);

  Relu_008 <= "100000000000" when truncate_008 < -2048 else
             "011111111111" when truncate_008 > 2047 else
             truncate_008(11 downto 0);

  Relu_009 <= "100000000000" when truncate_009 < -2048 else
             "011111111111" when truncate_009 > 2047 else
             truncate_009(11 downto 0);

  Relu_010 <= "100000000000" when truncate_010 < -2048 else
             "011111111111" when truncate_010 > 2047 else
             truncate_010(11 downto 0);

  Relu_011 <= "100000000000" when truncate_011 < -2048 else
             "011111111111" when truncate_011 > 2047 else
             truncate_011(11 downto 0);

  Relu_012 <= "100000000000" when truncate_012 < -2048 else
             "011111111111" when truncate_012 > 2047 else
             truncate_012(11 downto 0);

  Relu_013 <= "100000000000" when truncate_013 < -2048 else
             "011111111111" when truncate_013 > 2047 else
             truncate_013(11 downto 0);

  Relu_014 <= "100000000000" when truncate_014 < -2048 else
             "011111111111" when truncate_014 > 2047 else
             truncate_014(11 downto 0);

  Relu_015 <= "100000000000" when truncate_015 < -2048 else
             "011111111111" when truncate_015 > 2047 else
             truncate_015(11 downto 0);


--ReluSum <= ("0000000" & Relu_00) + ("0000000" & Relu_01)
--		+ ("0000000" & Relu_02) + ("0000000" & Relu_03)
--		+ ("0000000" & Relu_04) + ("0000000" & Relu_05)
--		+ ("0000000" & Relu_06) + ("0000000" & Relu_07)
--		+ ("0000000" & Relu_08) + ("0000000" & Relu_09)
--
--		+ ("0000000" & Relu_10) + ("0000000" & Relu_11)
--		+ ("0000000" & Relu_12) + ("0000000" & Relu_13)
--		+ ("0000000" & Relu_14) + ("0000000" & Relu_15)
--		+ ("0000000" & Relu_16) + ("0000000" & Relu_17)
--		+ ("0000000" & Relu_18) + ("0000000" & Relu_19)
--
--		+ ("0000000" & Relu_20) + ("0000000" & Relu_21)
--		+ ("0000000" & Relu_22) + ("0000000" & Relu_23)
--		+ ("0000000" & Relu_24) + ("0000000" & Relu_25)
--		+ ("0000000" & Relu_26) + ("0000000" & Relu_27)
--		+ ("0000000" & Relu_28) + ("0000000" & Relu_29)
--
--		+ ("0000000" & Relu_30) + ("0000000" & Relu_31);

ReluAll <= 
	reg_000_000 & reg_000_001 & reg_000_002 & reg_000_003 & reg_000_004 & reg_000_005 & reg_000_006 & reg_000_007 &
	reg_001_000 & reg_001_001 & reg_001_002 & reg_001_003 & reg_001_004 & reg_001_005 & reg_001_006 & reg_001_007 &
	reg_002_000 & reg_002_001 & reg_002_002 & reg_002_003 & reg_002_004 & reg_002_005 & reg_002_006 & reg_002_007 &
	reg_003_000 & reg_003_001 & reg_003_002 & reg_003_003 & reg_003_004 & reg_003_005 & reg_003_006 & reg_003_007 &
	reg_004_000 & reg_004_001 & reg_004_002 & reg_004_003 & reg_004_004 & reg_004_005 & reg_004_006 & reg_004_007 &
	reg_005_000 & reg_005_001 & reg_005_002 & reg_005_003 & reg_005_004 & reg_005_005 & reg_005_006 & reg_005_007 &
	reg_006_000 & reg_006_001 & reg_006_002 & reg_006_003 & reg_006_004 & reg_006_005 & reg_006_006 & reg_006_007 &
	reg_007_000 & reg_007_001 & reg_007_002 & reg_007_003 & reg_007_004 & reg_007_005 & reg_007_006 & reg_007_007 &
	reg_008_000 & reg_008_001 & reg_008_002 & reg_008_003 & reg_008_004 & reg_008_005 & reg_008_006 & reg_008_007 &
	reg_009_000 & reg_009_001 & reg_009_002 & reg_009_003 & reg_009_004 & reg_009_005 & reg_009_006 & reg_009_007 &
	reg_010_000 & reg_010_001 & reg_010_002 & reg_010_003 & reg_010_004 & reg_010_005 & reg_010_006 & reg_010_007 &
	reg_011_000 & reg_011_001 & reg_011_002 & reg_011_003 & reg_011_004 & reg_011_005 & reg_011_006 & reg_011_007 &
	reg_012_000 & reg_012_001 & reg_012_002 & reg_012_003 & reg_012_004 & reg_012_005 & reg_012_006 & reg_012_007 &
	reg_013_000 & reg_013_001 & reg_013_002 & reg_013_003 & reg_013_004 & reg_013_005 & reg_013_006 & reg_013_007 &
	reg_014_000 & reg_014_001 & reg_014_002 & reg_014_003 & reg_014_004 & reg_014_005 & reg_014_006 & reg_014_007 &
	reg_015_000 & reg_015_001 & reg_015_002 & reg_015_003 & reg_015_004 & reg_015_005 ; ---- reg_015_006 & reg_015_007 ;


ReluAllWrN <= not enb_007;

end RTL;

